package core;

import utils.exceptions.FileParseException;
import java.io.IOException;
import java.util.StringTokenizer;
import utils.io.BufferedReaderExtended;
import utils.UtilsIO;
import utils.UtilsLogging;
import utils.exceptions.SyntaxException;

/**
 *
 * @author Jason Koukas
 */
public class SUBBlock extends SubtitleBlock {

    public static final double FPS_MIN = 18.0;
    public static final double FPS_MAX = 72.0;
    public static final double[] POPULAR_FRAMERATES = new double[]{23.976, 25.0, 29.97};

    @Override
    protected void clearDataSubclass() {
    }

    @Override
    protected char getLinesDelimiter() {
        return '|';
    }

    @Override
    public SubtitleType getType() {
        return SubtitleType.SUB_TYPE;
    }

    @Override
    public String getTimestampString() {
        return "f-" + String.valueOf(m_timeStart);
    }

    @Override
    public String toStringRepresentation() {
        return "{" + m_timeStart + "}" + "{" + m_timeEnd + "}" + toString();
    }

    @Override
    public boolean readBlock(BufferedReaderExtended br) throws IOException, FileParseException {
        clearData(); //for safety because one might assume that the block's clear

        try {
            String line = UtilsIO.getNextLine(br, true);
            if (line == null) {
                return false;
            }

            //System.out.println(line);

            StringTokenizer st = new StringTokenizer(line, "{}", true);
            verifyNextTokenIs(st, "{");//{
            m_timeStart = verifyIntegerNextToken(st);
            verifyNextTokenIs(st, "}");//}
            verifyNextTokenIs(st, "{");//{
            m_timeEnd = verifyIntegerNextToken(st);
            verifyNextTokenIs(st, "}");//}

            //text += "\n"; //Den xreiazetai giati ta sub arxeia den exoun enter sto telos
            st = new StringTokenizer(getRemainingTokensAsLine(st), "|", false);
            m_subtitleLines.clear();
            while (st.hasMoreTokens()) {
                m_subtitleLines.add(st.nextToken());
            }
            return true;
        } catch (IOException ex) {
            clearData();
            throw ex;
        } catch (SyntaxException ex) {
            UtilsLogging.log(getClass(), ex);
            clearData();
            throw new FileParseException(ex.getMessage(), br.getLinesRead(), br.getLastReadLine());
        }
    }

    @Override
    public SubtitleBlock clone() {
        SUBBlock clone = new SUBBlock();
        cloneTo(clone);
        return clone;
    }
}
