package core;

import utils.exceptions.FileParseException;
import java.io.IOException;
import java.io.File;
import java.io.InputStreamReader;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import utils.io.BufferedReaderExtended;
import utils.UtilsIO;
import utils.exceptions.WarningException;

public class Subsync {

//    private static final Charset CHARSET_DEFAULT;
//
//    static {
//        Charset charset;
//        try {
//            charset = Charset.forName("ISO-8859-7");
//        } catch (UnsupportedCharsetException uce) {
//            charset = Charset.defaultCharset();
//        }
//        CHARSET_DEFAULT = charset;
//    }
    public static ArrayList<SubtitleBlock> readAllBlocks(File sourceFile, SubtitleType subFormat)
            throws IOException, FileParseException {
        BufferedReaderExtended br = null;
        try {
            br = new BufferedReaderExtended(new InputStreamReader(new FileInputStream(sourceFile)));

            ArrayList<SubtitleBlock> ret = new ArrayList<SubtitleBlock>();
            SubtitleBlock sourceBlock;
            do {
                switch (subFormat) {

                    case SUB_TYPE:
                        sourceBlock = new SUBBlock();
                        break;
                    //case SRT_TYPE:
                    default:
                        sourceBlock = new SRTBlock();
                        break;
                }
                if (!sourceBlock.readBlock(br)) {
                    break;
                }
                ret.add(sourceBlock);
            } while (true);
            return ret;
        } finally {
            UtilsIO.safelyCloseStream(br);
            //return null;
        }
    }

    public static void writeAllBlocks(File destFile, ArrayList<SubtitleBlock> blocks) throws IOException {
        PrintStream ps = null;
        try {
            ps = new PrintStream(new FileOutputStream(destFile));
            for (SubtitleBlock block : blocks) {
                block.writeBlock(ps);
            }
        } finally {
            UtilsIO.safelyCloseStream(ps);
            //return null;
        }
    }

    public static ArrayList<SubtitleBlock> synchronize(ArrayList<SubtitleBlock> blockList, SubtitleType sourceFormat, SubtitleType destFormat,
            int selectedStart, int selectedEnd, int correctStart, int correctEnd, double destFpsOrFirstIndex) throws WarningException { //correct values are observed
        //System.out.println(s2+" "+e2);

        boolean completedButWithWarning = false;
        //Ara otan einai apo sub se srt vazeis fps = 1.0 opote den peirazei giati
        //allhloanaireitai oti timh kai na valeis, akoma kai diaforh tou 1.0.
        //Vazeis 1 gia na arxizei apo kei sto srt.
        //MIPWS KAPOIA STIGMI NA KANEIS ENA README???????

        double a = (correctEnd - correctStart) / ((double) (selectedEnd - selectedStart));
        double b = correctStart - a * selectedStart;
        //System.out.println("a = "+a);
        //System.out.println("b = "+b);

        ArrayList<SubtitleBlock> blocks = new ArrayList<SubtitleBlock>();
        SubtitleBlock destBlock;
        int blockIndex = 0;
        for (SubtitleBlock sourceBlock : blockList) {
            //update block values

            double destTimeStart = sourceBlock.getStart() * a + b;
            double destTimeEnd = sourceBlock.getEnd() * a + b;
            if (destTimeStart < 0 || destTimeEnd < 0) {
                completedButWithWarning = true;
            }

            if (destFormat == SubtitleType.SRT_TYPE) {
                int destIndex;
                if (sourceBlock instanceof SRTBlock && ((SRTBlock) sourceBlock).getIndex() == SRTBlock.SUBTITLE_INDEX_SPECIAL) {
                    destIndex = SRTBlock.SUBTITLE_INDEX_SPECIAL;
                    --blockIndex;//Gia na parameinei idio sto epomeno?..
                } else {
                    destIndex = blockIndex + (int) destFpsOrFirstIndex;
                }
                destBlock = cloneToSRT(sourceBlock, (int) destTimeStart, (int) destTimeEnd, destIndex);
                //translate them to other type
            } else {
                destBlock = cloneToSUB(sourceBlock, (int) millisecondsToFrame(destTimeStart, destFpsOrFirstIndex),
                        (int) millisecondsToFrame(destTimeEnd, destFpsOrFirstIndex));
            }
            blocks.add(destBlock);
            ++blockIndex;
        }

        if (completedButWithWarning) {
            throw new WarningException(blocks, "Synchronization complete but some negative timestamps occured.");
        }
        return blocks;
    }

    public static double millisecondsToFrame(double millis, double fps) {
        return (millis * fps / TimeConstants.MILLIS_PER_SECOND);
    }

    public static double frameToMilliseconds(double frame, double fps) {
        return frame * TimeConstants.MILLIS_PER_SECOND / fps;
    }

    private static SRTBlock cloneToSRT(SubtitleBlock sourceBlock, int destTimeStart, int destTimeEnd, int destIndex) {
        SRTBlock destBlock = new SRTBlock();
        cloneBaseVariables(sourceBlock, destBlock, destTimeStart, destTimeEnd);
        destBlock.setIndex(destIndex);
        return destBlock;
    }

    private static SUBBlock cloneToSUB(SubtitleBlock sourceBlock, int destTimeStart, int destTimeEnd) {
        SUBBlock destBlock = new SUBBlock();
        cloneBaseVariables(sourceBlock, destBlock, destTimeStart, destTimeEnd);
        return destBlock;
    }

    private static void cloneBaseVariables(SubtitleBlock sourceBlock, SubtitleBlock destBlock, int destTimeStart, int destTimeEnd) {
        destBlock.setStart(destTimeStart);
        destBlock.setEnd(destTimeEnd);
        ArrayList<String> blockLines = sourceBlock.getSubtitleLines();
        for (String blockLine : blockLines) {
            destBlock.addLine(blockLine);
        }
    }
}
