package core;

import utils.exceptions.FileParseException;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.StringTokenizer;
import utils.io.BufferedReaderExtended;
import utils.UtilsIO;
import utils.exceptions.SyntaxException;

public abstract class SubtitleBlock {

    protected static final String FORMAT_ERR = "Input format error";
    protected int m_timeStart, m_timeEnd;
    protected ArrayList<String> m_subtitleLines;
    //protected ArrayList<String> strings;
    //protected IntegerEditable nextIndex;

    public SubtitleBlock() {
        m_subtitleLines = new ArrayList<String>();
    }

    public void setStart(int timeStart) {
        m_timeStart = timeStart;
    }

    public int getStart() {
        return m_timeStart;
    }

    public void setEnd(int timeEnd) {
        this.m_timeEnd = timeEnd;
    }

    public int getEnd() {
        return m_timeEnd;
    }

    public ArrayList<String> getSubtitleLines() {
        return this.m_subtitleLines;
    }

    public void addLine(String text) {
        this.m_subtitleLines.add(text);
    }

    public void clearData() {
        m_timeStart = 0;
        m_timeEnd = 0;
        m_subtitleLines.clear();
        clearDataSubclass();
    }

    @Override
    public String toString() {
        if (m_subtitleLines.isEmpty()) {
            return "";
        }
        String rep = m_subtitleLines.get(0);
        for (int i = 1; i < m_subtitleLines.size(); i++) {
            rep = rep + getLinesDelimiter() + m_subtitleLines.get(i);
        }
        return rep;
    }

    public void writeBlock(PrintStream ps)
            throws IOException {
        ps.println(toStringRepresentation());
    }

    protected abstract void clearDataSubclass();

    protected abstract char getLinesDelimiter();

    public abstract SubtitleType getType();

    public abstract boolean readBlock(BufferedReaderExtended br) throws IOException, FileParseException;

    public abstract String getTimestampString();

    public abstract String toStringRepresentation();

    //Methods to be re-implemented in subclasses.
    @Override
    public abstract SubtitleBlock clone();

    //Protected Helpful Methods
    protected void cloneTo(SubtitleBlock clone) {
        clone.m_timeStart = m_timeStart;
        clone.m_timeEnd = m_timeEnd;
        clone.m_subtitleLines = new ArrayList<String>(m_subtitleLines);
    }

    protected static String demandToken(StringTokenizer st) throws SyntaxException {
        if (!st.hasMoreTokens()) {
            throw new SyntaxException(FORMAT_ERR);
        }
        return st.nextToken();
    }

    protected static void verifyNextTokenIs(StringTokenizer st, String valueExpected) throws SyntaxException {
        if (!demandToken(st).equals(valueExpected)) {
            throw new SyntaxException(FORMAT_ERR);
        }
    }

//    protected static void verifyNextTokenIsNoCase(StringTokenizer st, String valueExpected) throws FormattingException {
//        if (!demandToken(st).equalsIgnoreCase(valueExpected)) {
//            throw new FormattingException(FORMAT_ERR);
//        }
//    }
    protected static int verifyIntegerNextToken(StringTokenizer st) throws SyntaxException {
        return verifyInteger(demandToken(st));
    }

    protected static int verifyInteger(String value) throws SyntaxException {
        try {
            int val = parseIntWithTrim(value);
            if (val < 0) {
                throw new SyntaxException(FORMAT_ERR);
            }
            return val;
        } catch (NumberFormatException ex) {
            throw new SyntaxException(FORMAT_ERR);
        }
    }

    protected static int parseIntWithTrim(String value) {
        return Integer.parseInt(value.trim());
    }

    protected static String getRemainingTokensAsLine(StringTokenizer st) {
        String line = "";
        while (st.hasMoreTokens()) {
            line += st.nextToken();
        }
        return line;
    }

    protected static String demandLine(BufferedReaderExtended brExtend, boolean ignoreEmpty) throws IOException, SyntaxException {
        String line = UtilsIO.getNextLine(brExtend, ignoreEmpty);
        if (line == null) {
            throw new SyntaxException(FORMAT_ERR);
        }
        return line;
    }
}
