package gui;

import core.SubtitleBlock;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.border.TitledBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import utils.UtilsUI;
import utils.UtilsImages;
import utils.UtilsLookAndFeel;

/**
 *
 * @author Jason Koukas
 */
public class SearchPanel extends JPanel {

    private static final int NO_SELECTION = -1;
    private static final int SEARCH_NOT_FOUND = -1;
    private static final int NO_LAST_SEARCH_RESULT = -2;
    private static final Color COLOR_TEXT_FIELD_DEFAULT = Color.white;
    private static final Color COLOR_TEXT_FIELD_NOT_FOUND = new Color(0xFFDDDD);
    //private static final Icon SEARCH_ICON = new ImageIcon(SearchPanel.class.getResource("/resources/icons/search-32.png"));
    private static final Icon ICON_PREV = UtilsImages.loadImageIcon("/resources/icons/findPrev-32.png");
    private static final Icon ICON_NEXT = UtilsImages.loadImageIcon("/resources/icons/findNext-32.png");
    //
    private int m_selectionIndex;
    private int m_lastSearchResultCode;
    private String m_lastSearchString;
    private JTextField m_findField;
    //
    //private JButton searchButt;
    private JButton m_findPrevButt;
    private JButton m_findNextButt;
    //
    private SyncPanel m_syncPanel;

    public SearchPanel(SyncPanel displayPane) {
        m_syncPanel = displayPane;
        TitledBorder fieldBorder = UtilsLookAndFeel.getTitledBorder("Search");

        m_findField = new JTextField();
        m_findField.getDocument().addDocumentListener(new DocumentListener() {

            public void insertUpdate(DocumentEvent e) {
                findNew();
                m_lastSearchString = m_findField.getText();
            }

            public void removeUpdate(DocumentEvent e) {
                findNew();
                m_lastSearchString = m_findField.getText();
            }

            public void changedUpdate(DocumentEvent e) {
            }
        });
        //m_findField.setBorder();

        JPanel textFieldWrapper = new JPanel(new BorderLayout());
        textFieldWrapper.setBorder(fieldBorder);
        textFieldWrapper.add(m_findField, BorderLayout.CENTER);


        JToolBar findToolbar = UtilsUI.getToolBar();
        findToolbar.setBorder(BorderFactory.createEmptyBorder(5, 2, 1, 3));

        //searchButt = new JButton("", SEARCH_ICON);
        //searchButt.setToolTipText("Find phrase");
        //searchButt.setBackground(SynchronizerGUI.MAIN_COLOR);
        //searchButt.addActionListener(this);

        ActionListener buttonListener = new ActionListener() {

            public void actionPerformed(ActionEvent evt) {
                Object src = evt.getSource();
                if (src == m_findNextButt) {
                    findNext();
                } else if (src == m_findPrevButt) {
                    findPrev();
                }
            }
        };

        m_findPrevButt = new JButton("", ICON_PREV);
        m_findPrevButt.setToolTipText("Find previous");
        m_findPrevButt.addActionListener(buttonListener);

        m_findNextButt = new JButton("", ICON_NEXT);
        m_findNextButt.setToolTipText("Find next");
        m_findNextButt.addActionListener(buttonListener);

        //findToolbar.add(searchButt);
        findToolbar.add(m_findPrevButt);
        findToolbar.add(m_findNextButt);

        //setPreferredSize(new Dimension(100, 30));
        setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
        add(textFieldWrapper);
        add(findToolbar);

        //reset() needs to be called but is called from outside
    }

    public void reset() {
        m_selectionIndex = NO_SELECTION;
        m_lastSearchResultCode = NO_LAST_SEARCH_RESULT;
        m_lastSearchString = "";

        m_findField.setText("");
        m_findField.setBackground(COLOR_TEXT_FIELD_DEFAULT);
        m_findNextButt.setEnabled(false);
        m_findPrevButt.setEnabled(false);
    }

    public void setSelectedDisplayArea(SubtitleDisplayArea displayArea) {
        m_selectionIndex = m_syncPanel.getSubtitlesPane().getSubtitles().indexOf(displayArea);
    }

    public boolean findNext() {
        return findAgain(true, true);
    }

    public boolean findPrev() {
        return findAgain(false, true);
    }

    @Override
    public void requestFocus() {
        m_findField.requestFocus();
    }

    /**
     * Private Helpful Methods
     */
    private void findNew() {
        findAgain(true, false);
    }

    private boolean findAgain(boolean forward, boolean fromFindPrevNext) {
        //JButton sameDirection, oppositeDirection;
        int indexFrom, indexFirst;
        String phrase = m_findField.getText();

        if (fromFindPrevNext) {
            if (m_lastSearchResultCode < 0) { //&& m_lastSearchString.equals(phrase)) {
                Toolkit.getDefaultToolkit().beep();
                return false;
            }
        } else {
//        if (phrase.length() == 0) {
//            resetPrivate();
//            return false;
//        }

            if (phrase.indexOf(m_lastSearchString) >= 0) {
                if (m_lastSearchResultCode == SEARCH_NOT_FOUND && !"".equals(m_lastSearchString)) {
                    Toolkit.getDefaultToolkit().beep();
                    return false;
                }
            } else if (m_lastSearchString.indexOf(phrase) >= 0) {
                if (m_lastSearchResultCode != SEARCH_NOT_FOUND && !"".equals(phrase)) {
                    m_syncPanel.selectFoundSubtitle(m_lastSearchResultCode); //Used only to scroll in fact..
                    return true;
                }
            }
        }

        indexFrom = searchFirstIndex(forward, fromFindPrevNext);
        if (forward) {
            indexFirst = 0;
            //sameDirection = m_findNextButt;
            //oppositeDirection = m_findPrevButt;
        } else {
            indexFirst = m_syncPanel.getSubtitlesPane().getSubtitles().size() - 1;
            //sameDirection = m_findPrevButt;
            //oppositeDirection = m_findNextButt;
        }

        int code = find(phrase, indexFrom, forward);
        if (code == SEARCH_NOT_FOUND && indexFrom != indexFirst) {
            m_lastSearchResultCode = find(phrase, indexFirst, forward);
            //sameDirection.setEnabled(false);
        } else {
            m_lastSearchResultCode = code;
            //oppositeDirection.setEnabled(true);
        }

        if (m_lastSearchResultCode == SEARCH_NOT_FOUND) {
            m_selectionIndex = indexFrom;
            m_syncPanel.clearFoundSubtitle();
            if ("".equals(phrase)) {
                m_findField.setBackground(COLOR_TEXT_FIELD_DEFAULT);
            } else {
                m_findField.setBackground(COLOR_TEXT_FIELD_NOT_FOUND);
                Toolkit.getDefaultToolkit().beep();
            }
            m_findNextButt.setEnabled(false);
            m_findPrevButt.setEnabled(false);
            return false;
        } else {
            m_syncPanel.selectFoundSubtitle(m_lastSearchResultCode);
            m_findField.setBackground(COLOR_TEXT_FIELD_DEFAULT);
            m_findNextButt.setEnabled(true);
            m_findPrevButt.setEnabled(true);
            return true;
        }
    }

    private int searchFirstIndex(boolean forward, boolean fromFindPrevNext) {
        if (m_selectionIndex != NO_SELECTION) {
            int tempSelection;
            if (fromFindPrevNext && m_selectionIndex == m_lastSearchResultCode) {
                tempSelection = getFollowingIndex(m_selectionIndex, forward);
            } else {
                tempSelection = m_selectionIndex;
            }
            m_selectionIndex = NO_SELECTION;
            return tempSelection;
        } else {
            if (m_lastSearchResultCode != NO_LAST_SEARCH_RESULT) {
                if (fromFindPrevNext) {
                    return getFollowingIndex(m_lastSearchResultCode, forward);
                }
                return m_lastSearchResultCode;
            } else {
                if (forward) {
                    return 0;
                } else {
                    return m_syncPanel.getSubtitlesPane().getSubtitles().size() - 1;
                }
            }
        }
    }

    private int getFollowingIndex(int currentIndex, boolean forward) {
        if (forward) {
            if (currentIndex == m_syncPanel.getSubtitlesPane().getSubtitles().size() - 1) {
                return 0;
            } else {
                return currentIndex + 1;
            }
        } else {
            if (currentIndex == 0) {
                return m_syncPanel.getSubtitlesPane().getSubtitles().size() - 1;
            }
            return currentIndex - 1;
        }
    }

    private int find(String phrase, int indexFrom, boolean forward) {
        phrase = phrase.toUpperCase();
        if (!"".equals(phrase)) {
            ArrayList<SubtitleDisplayArea> subtitles = m_syncPanel.getSubtitlesPane().getSubtitles();
            if (forward) {
                for (int i = indexFrom; i < subtitles.size(); i++) {
                    SubtitleBlock theBlock = subtitles.get(i).getSubtitleBlock();
                    if (findPhrase(phrase, theBlock)) {
                        return i;
                    }
                }
            } else {
                for (int i = indexFrom; i >= 0; i--) {
                    SubtitleBlock theBlock = subtitles.get(i).getSubtitleBlock();
                    if (findPhrase(phrase, theBlock)) {
                        return i;
                    }
                }
            }
        }
        return SEARCH_NOT_FOUND;
    }

    private boolean findPhrase(String phraseInCaps, SubtitleBlock block) {
        ArrayList<String> subtitleLines = block.getSubtitleLines();
        for (String subtitleLine : subtitleLines) {
            if (subtitleLine.toUpperCase().indexOf(phraseInCaps) != -1) {
                return true;
            }
        }
        return false;
    }
}
