package gui;

import core.SubtitleBlock;
import java.awt.BorderLayout;

import java.util.ArrayList;

import java.awt.Font;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import utils.UtilsLookAndFeel;

/**
 *
 * @author Jason Koukas
 */
public class SubtitleDisplayArea extends JComponent {

    public static final Color COLOR_DEFAULT_BACKGROUND = Color.WHITE;
    public static final Color COLOR_TIMESTAMP_BACKGROUND = new Color(0xDDEEFF);
    private static final Color COLOR_LETTERS = Color.BLACK;
    private static final Color COLOR_BORDER = Color.lightGray;
    //
    private static final Font FIELD_FONT = UtilsLookAndFeel.deriveFont(UtilsLookAndFeel.FONT, 1.3f); //A custom Font which we use for the field
    private static final Dimension TIMESTAMP_DIMENSIONS = new Dimension(100, 10);
    //
    private static final int MARGIN_HORIZONTAL = 5;
    private static final int MARGIN_TOP = 1;
    private static final int MARGIN_BOTTOM = 4;
    //
    private SubtitleBlock m_subtitleBlock; //The subtitle associated with the field
    private SubtitleTextArea m_displayText;
    private SubtitleTimestampArea m_displayTimestamp;

    private class SubtitleTextArea extends JPanel {

        public SubtitleTextArea() {
            setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
            setBorder(BorderFactory.createEmptyBorder(MARGIN_TOP, MARGIN_HORIZONTAL, MARGIN_BOTTOM, MARGIN_HORIZONTAL));

            ArrayList<String> strings = m_subtitleBlock.getSubtitleLines();
            for (String line : strings) {
                JLabel labelLine = new JLabel(line);
                labelLine.setFont(FIELD_FONT);
                labelLine.setForeground(COLOR_LETTERS);
                add(labelLine);
            }
        }

        @Override
        public void paintComponent(Graphics g) {
            super.paintComponent(g);
            g.setColor(COLOR_BORDER);
            int x = getWidth() - 1, y = getHeight() - 1;
            g.drawLine(1, y, x - 1, y);
            g.drawLine(0, 0, 0, y);
            g.drawLine(x, 0, x, y);
        }
    }

    private class SubtitleTimestampArea extends JLabel {

        public SubtitleTimestampArea() {
            super(m_subtitleBlock.getTimestampString());
            setHorizontalAlignment(CENTER);
            setBorder(BorderFactory.createEmptyBorder(MARGIN_TOP, MARGIN_HORIZONTAL, MARGIN_BOTTOM, MARGIN_HORIZONTAL));
            setPreferredSize(TIMESTAMP_DIMENSIONS);
            setToolTipText(getText());
            setFont(FIELD_FONT);

            setForeground(COLOR_LETTERS);
            setOpaque(true);
        }

        @Override
        public void paintComponent(Graphics g) {
            super.paintComponent(g);
            g.setColor(COLOR_BORDER);
            int x = getWidth() - 1, y = getHeight() - 1;

            g.drawLine(0, 0, 0, y);
            g.drawLine(1, y, x, y);
        }
    }

    public SubtitleDisplayArea(SubtitleBlock theBlock) {
        setLayout(new BorderLayout());
        m_subtitleBlock = theBlock;

        m_displayTimestamp = new SubtitleTimestampArea();
        m_displayText = new SubtitleTextArea();

        setBackground(COLOR_DEFAULT_BACKGROUND);
        setTimestampBackground(COLOR_TIMESTAMP_BACKGROUND);
        add(BorderLayout.WEST, m_displayTimestamp);
        add(BorderLayout.CENTER, m_displayText);
    }

    /**
     * Returns the subtitle block associated with that field
     * @return the subtitle block associated with the field
     */
    public SubtitleBlock getSubtitleBlock() {
        return m_subtitleBlock;
    }

    public void setTimestampBackground(Color background) {
        m_displayTimestamp.setBackground(background);
    }

    @Override
    public void setBackground(Color background) {
        super.setBackground(background);
        m_displayText.setBackground(background);

        //System.out.println(m_displayText.getBackground());
    }
}
