package gui;

import core.SubtitleBlock;

import java.util.ArrayList;
import java.awt.Color;
import javax.swing.BoxLayout;

import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import utils.UtilsUI;

/**
 *
 * @author Jason Koukas
 */
public class SubtitlesPane extends JPanel implements UtilsUI.RecursiveBackgroundIgnore {

    protected static final Color COLOR_DEFAULT_AREA_BACKGROUND = SubtitleDisplayArea.COLOR_DEFAULT_BACKGROUND;
    protected static final int SCROLL_HEIGHT = 50;
    protected JScrollPane m_scrollPane; //The Panel where all the SubtitleShowAreas are added to
    protected JScrollBar m_scrollBarVertical;
    protected ArrayList<SubtitleDisplayArea> m_subtitles; //An ArrayList list containing the subtitles that were added as we can't access them through the Panel

    /**
     * Construction of a new SubtitlesPane
     */
    public SubtitlesPane() {
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

        m_scrollPane = new JScrollPane(this);
        m_scrollBarVertical = m_scrollPane.getVerticalScrollBar();
        m_scrollBarVertical.setUnitIncrement(SCROLL_HEIGHT);
        m_scrollPane.setBackground(COLOR_DEFAULT_AREA_BACKGROUND);

        setBackground(COLOR_DEFAULT_AREA_BACKGROUND);

        m_subtitles = new ArrayList<SubtitleDisplayArea>();
    }

    public JScrollPane getScrollPaneContainer() {
        return m_scrollPane;
    }

    public ArrayList<SubtitleDisplayArea> getSubtitles() {
        return m_subtitles;
    }

    //public void addSubtitle(SubtitleBlock theBlock) {
    //    addSubtitleWithoutRevalidation(theBlock);
    //    revalidate();
    //    repaint();
    //}
    /**
     * Clears all the fields
     */
    public void reset(ArrayList<SubtitleBlock> subtitles) {
        removeAll();
        m_subtitles.clear();

        for (SubtitleBlock subtitle : subtitles) {
            addSubtitleWithoutRevalidation(subtitle);
        }
        revalidate();
        repaint();
        //At the end - just in case
        m_scrollBarVertical.setValue(0);
    }

    public void scrollToBlock(SubtitleDisplayArea displayArea) {
        m_scrollBarVertical.setValue(displayArea.getY());
    }

    public void scrollToBlock(int blockIndex) {
        scrollToBlock(m_subtitles.get(blockIndex));
    }

    /**
     * Private Helpful Methdos
     */
//    private void setInnerSize(Dimension size) {
//        setMinimumSize(size); //Here we call the standard methods of swing objects
//        setPreferredSize(size);
//        setMaximumSize(size);
//    }
    private void addSubtitleWithoutRevalidation(SubtitleBlock subBlock) {
        SubtitleDisplayArea subArea = new SubtitleDisplayArea(subBlock);
        m_subtitles.add(subArea); //Creates a new PlayerInfoPane and adds it to the list
        add(subArea);
    }
}
