/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package gui;

import core.SRTBlock;
import core.SUBBlock;
import core.Subsync;
import core.SubtitleBlock;
import utils.exceptions.FileParseException;
import core.SubtitleType;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;
import javax.swing.border.Border;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.text.NumberFormatter;
import utils.UtilsFiles;
import utils.UtilsUI;
import utils.UtilsIO;
import utils.UtilsLogging;
import utils.UtilsImages;
import utils.UtilsLookAndFeel;
import utils.dnd.TransferHandlerCustomExtend;
import utils.exceptions.InvalidParameterValuesException;
import utils.exceptions.WarningException;
import utils.ui.HTMLViewer;
import utils.ui.JSpinnerEditable;
import utils.ui.JSpinnerEditable.SpinnerModelInt;
import utils.ui.LabelledWrapper;
import utils.ui.SpinnerModelDoublePredefinedValuesCycle;

/**
 *
 * Date: 12 Μαρ 2012
 * @author Jason Koukas
 */
public class SyncFrame extends JFrame {

    private static final String APP_VERSION_STRING = "v.1.0.2"; //Change Subsync.html, Launch4j, refreshPublicHtml+.bat
    private static final String APP_TITLE = "Subsync";
    private static final String APP_TITLE_LONG = APP_TITLE + " Subtitles Synchronizer";
    //
    private static final boolean SHOW_WARNING_ON_OPEN_NEW_FILE = false;
    private static final KeyStroke KEY_STROKE_SETTINGS = KeyStroke.getKeyStroke(KeyEvent.VK_F11, 0);
    //
    private static final DataFlavor[] SUPPORTED_FLAVORS_FILES = new DataFlavor[]{DataFlavor.javaFileListFlavor};
    private static final String[] SUBTITLES_FILTER_EXTENSIONS;
    private static final FileNameExtensionFilter[] SUBTITLE_FILES_OPEN_FILTERS;
    private static final FileNameExtensionFilter[] SUBTITLE_FILES_SAVE_FILTERS;
    //
    private static final ImageIcon ICON_APPLICATION_LOGO = UtilsImages.loadImageIcon("/resources/icons/appLogo-48.png");
    private static final Icon ICON_MENU_FILE = UtilsImages.loadImageIcon("/resources/icons/folder-16.png");
    private static final Icon ICON_MENU_OPEN = UtilsImages.loadImageIcon("/resources/icons/openFile-16.png");
    private static final Icon ICON_MENU_SAVE = UtilsImages.loadImageIcon("/resources/icons/save-16.png");
    private static final ImageIcon ICON_MENU_SETTINGS = UtilsImages.loadImageIcon("/resources/icons/gear-16.png");
    //
    private static final Icon ICON_MENU_SEARCH = UtilsImages.loadImageIcon("/resources/icons/search-16.png");
    private static final Icon ICON_MENU_FIND_NEXT = UtilsImages.getScaledImageIcon(UtilsImages.loadImageIcon("/resources/icons/findNext-32.png"), 16, 16);
    private static final Icon ICON_MENU_FIND_PREV = UtilsImages.getScaledImageIcon(UtilsImages.loadImageIcon("/resources/icons/findPrev-32.png"), 16, 16);
    //
    private static final Icon ICON_MENU_HELP = UtilsImages.loadImageIcon("/resources/icons/help-16.png");
    private static final ImageIcon ICON_MENU_GUIDE = UtilsImages.loadImageIcon("/resources/icons/helpContents-16.png");
    private static final ImageIcon ICON_MENU_ABOUT = UtilsImages.loadImageIcon("/resources/icons/info-16.png");
    //
    private static final String EXTENSION_SRT = "srt";
    private static final String EXTENSION_SUB = "sub";
    //

    static {
        SubtitleType[] subtitleTypes = SubtitleType.values();
        SUBTITLES_FILTER_EXTENSIONS = new String[subtitleTypes.length];
        int i = 0;
        for (SubtitleType subtitleType : subtitleTypes) {
            SUBTITLES_FILTER_EXTENSIONS[i] = subtitleType.getLowerCaseExtension();
            i++;
        }
        SUBTITLE_FILES_OPEN_FILTERS = new FileNameExtensionFilter[]{
                    UtilsFiles.createFileNameExtensionFilter("Subtitle Files", new String[]{EXTENSION_SRT, EXTENSION_SUB})
                };
        SUBTITLE_FILES_SAVE_FILTERS = new FileNameExtensionFilter[]{
                    UtilsFiles.createFileNameExtensionFilter("SRT Files", new String[]{EXTENSION_SRT}),
                    UtilsFiles.createFileNameExtensionFilter("SUB Files", new String[]{EXTENSION_SUB})
                };
    }
    //
    protected SyncPanel m_mainPanel;
    protected JButton m_configureSettings;
    protected JPanel m_panelConfiguration;
    protected JMenuItem m_menuItemExport;
    protected JSpinnerEditable m_spinnerSRTStartingIndex;//They are related to m_menuItemExport, in a way
    protected JSpinnerEditable m_spinnerSUBFps;
    private File m_fileLocationCurrent;

    private class TransferHandlerOpenFile extends TransferHandlerCustomExtend {

        public TransferHandlerOpenFile() {
            super(m_mainPanel, SUPPORTED_FLAVORS_FILES, true);
        }

        @Override
        public boolean validateTransferData(Transferable transferable, Object transferData, DataFlavor flavorPreferred) {
            //....
            File firstFile = ((List<File>) transferData).get(0);
            return SUBTITLE_FILES_OPEN_FILTERS[0].accept(firstFile);
        }

        @Override
        public boolean importData(Component comp, Transferable transferable, Object transferData, DataFlavor usedFlavor) {
            File firstFile = ((List<File>) transferData).get(0);
            return loadSubtitleFile(firstFile); //Extra check for null.. but avoid duplicate code..
        }
    }

    public SyncFrame() {
        super(APP_TITLE_LONG);
        setIconImage(ICON_APPLICATION_LOGO.getImage());

        m_mainPanel = new SyncPanel();
        initPanelConfiguration();
        {
            JToolBar toolBar = UtilsUI.getToolBar();
            JButton buttonSettings = toolBar.add(new AbstractAction("", ICON_MENU_SETTINGS) {

                public void actionPerformed(ActionEvent e) {
                    showSynchronizationSettings();
                }
            });
            buttonSettings.setToolTipText("Synchronization settings (" + KEY_STROKE_SETTINGS.toString().substring(8) + ")");
            JPanel panelDummy = new JPanel(new FlowLayout(FlowLayout.RIGHT, 0, 0));
            panelDummy.add(toolBar);
            m_mainPanel.getToolBar().add(panelDummy);
        }

        new TransferHandlerOpenFile();

        getContentPane().add(m_mainPanel);
        setJMenuBar(createMenus());
        UtilsUI.setBackgroundRecursive(this, UtilsLookAndFeel.COLOR_BACKGROUND_MAIN);
    }

    public boolean loadSubtitleFile(File subtitleFile) {
        if (subtitleFile != null && passWarnForOpenNewFile()) {
            return openSubtitleFile(subtitleFile);
        }
        return false;
    }

    @Override
    public void requestFocus() {
        m_mainPanel.requestFocus();
    }

    /**
     * Private Helpful Methods
     */
    private void initPanelConfiguration() {
        m_panelConfiguration = new JPanel();
        m_panelConfiguration.setBorder(BorderFactory.createEmptyBorder(10, 16, 10, 16));
        Dimension labelDim = new Dimension(323, 20);
        Dimension spinnersDim = new Dimension(58, 25);
        Border spinnersEmptyBorder = BorderFactory.createEmptyBorder(0, 0, 0, 6);
        m_panelConfiguration.setLayout(new BoxLayout(m_panelConfiguration, BoxLayout.Y_AXIS));
        {
            m_spinnerSRTStartingIndex = new JSpinnerEditable(new SpinnerModelInt(SRTBlock.FIRST_SUBTITLE_INDEX_MIN, SRTBlock.FIRST_SUBTITLE_INDEX_MAX, SRTBlock.FIRST_SUBTITLE_INDEX_DEFAULT));
            m_spinnerSRTStartingIndex.setPreferredSize(new Dimension(50, 20));
            m_spinnerSRTStartingIndex.getFormattedTextField().setHorizontalAlignment(SwingConstants.CENTER);
            //m_spinnerSRTStartingIndex.setTextFieldEditFormatter(new NumberFormatter());
            JPanel spinnerWrapper = UtilsUI.getWrapperPanel(m_spinnerSRTStartingIndex, false);
            spinnerWrapper.setPreferredSize(spinnersDim);
            spinnerWrapper.setBorder(spinnersEmptyBorder);

            LabelledWrapper panelSRTIndex = new LabelledWrapper(spinnerWrapper, "<html>Starting subtitle index in case of SRT output file <b>(default is " + SRTBlock.FIRST_SUBTITLE_INDEX_DEFAULT + ")</b></html>");
            panelSRTIndex.setBorder(UtilsLookAndFeel.getTitledBorder("SRT starting index"));
            panelSRTIndex.getWrapperLabel().setPreferredSize(labelDim);

            m_panelConfiguration.add(panelSRTIndex);
        }

        {
            JPanel panelEmptyBorder = new JPanel();
            panelEmptyBorder.setPreferredSize(new Dimension(50, 3));
            m_panelConfiguration.add(panelEmptyBorder);
        }

        {
            m_spinnerSUBFps = new JSpinnerEditable(new SpinnerModelDoublePredefinedValuesCycle(SUBBlock.FPS_MIN, SUBBlock.FPS_MAX, SUBBlock.POPULAR_FRAMERATES[0], SUBBlock.POPULAR_FRAMERATES));
            m_spinnerSUBFps.setPreferredSize(new Dimension(58, 20));
            m_spinnerSUBFps.getFormattedTextField().setHorizontalAlignment(SwingConstants.CENTER);
            m_spinnerSUBFps.setTextFieldEditFormatter(new NumberFormatter(new DecimalFormat()));
            m_spinnerSUBFps.addChangeListener(new ChangeListener() {

                @Override
                public void stateChanged(ChangeEvent e) {
                    m_mainPanel.updateVideoFramerate((Double) m_spinnerSUBFps.getValue());
                }
            });
            m_mainPanel.updateVideoFramerate((Double) m_spinnerSUBFps.getValue());//for the first time
            JPanel spinnerWrapper = UtilsUI.getWrapperPanel(m_spinnerSUBFps, false);
            spinnerWrapper.setPreferredSize(spinnersDim);
            spinnerWrapper.setBorder(spinnersEmptyBorder);

            LabelledWrapper panelFps = new LabelledWrapper(spinnerWrapper, "<html>Video framerate in case of SUB output file <b>(default is " + SUBBlock.POPULAR_FRAMERATES[0] + ")</b></html>");
            panelFps.setBorder(UtilsLookAndFeel.getTitledBorder("SUB fps"));
            panelFps.getWrapperLabel().setPreferredSize(labelDim);

            m_panelConfiguration.add(panelFps);
        }
        UtilsUI.setBackgroundRecursive(m_panelConfiguration, Color.white);
    }

    private JMenuBar createMenus() {
        JMenuBar bar = new JMenuBar();
        {
            JMenu menuFile = new JMenu("File");
            menuFile.setIcon(ICON_MENU_FILE);

            JMenuItem menuItemOpen = new JMenuItem(new AbstractAction("Open...", ICON_MENU_OPEN) {

                public void actionPerformed(ActionEvent e) {
                    if (!passWarnForOpenNewFile()) {
                        return;
                    }
                    File openFile = UtilsFiles.getOpenFileDialog(SyncFrame.this, SUBTITLE_FILES_OPEN_FILTERS, m_fileLocationCurrent);
                    if (openFile != null) { //Null here comes from 'cancel'..
                        openSubtitleFile(openFile);
                    }
                }
            });
            menuItemOpen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_DOWN_MASK));
            menuFile.add(menuItemOpen);

            m_menuItemExport = new JMenuItem(new AbstractAction("Save synchronization as...", ICON_MENU_SAVE) {

                public void actionPerformed(ActionEvent e) {
                    try {
                        m_mainPanel.requestFocus();
                        m_mainPanel.verifySynchronizationValuesValid();
                        File saveFile = UtilsFiles.getSaveFileDialog(SyncFrame.this, SUBTITLE_FILES_SAVE_FILTERS,
                                new File(UtilsFiles.changeExtension(m_fileLocationCurrent.getAbsolutePath(), EXTENSION_SRT)));
                        if (saveFile != null) {
                            saveSubtitleFile(saveFile);
                        }
                    } catch (InvalidParameterValuesException iae) {
                        UtilsUI.showErrorMessage(SyncFrame.this, iae.getMessage());
                    }
                }
            });
            m_menuItemExport.setEnabled(false);
            m_menuItemExport.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_DOWN_MASK));
            menuFile.add(m_menuItemExport);

            menuFile.addSeparator();

            JMenuItem menuItemSettings = new JMenuItem(new AbstractAction("Synchronization settings...", ICON_MENU_SETTINGS) {

                public void actionPerformed(ActionEvent e) {
                    showSynchronizationSettings();
                }
            });
            menuItemSettings.setAccelerator(KEY_STROKE_SETTINGS);
            menuFile.add(menuItemSettings);

            bar.add(menuFile);
        }

        {
            JMenu menuSearch = new JMenu("Search");
            menuSearch.setIcon(ICON_MENU_SEARCH);

            JMenuItem menuItemFind = new JMenuItem(new AbstractAction("Find", ICON_MENU_SEARCH) {

                public void actionPerformed(ActionEvent e) {
                    m_mainPanel.requestFocus();
                }
            });
            menuItemFind.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, InputEvent.CTRL_DOWN_MASK));
            menuSearch.add(menuItemFind);

            JMenuItem menuItemFindNext = new JMenuItem(new AbstractAction("Find Next", ICON_MENU_FIND_NEXT) {

                public void actionPerformed(ActionEvent e) {
                    m_mainPanel.findNext();
                }
            });
            menuItemFindNext.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F3, 0));
            menuSearch.add(menuItemFindNext);

            JMenuItem menuItemFindPrev = new JMenuItem(new AbstractAction("Find Previous", ICON_MENU_FIND_PREV) {

                public void actionPerformed(ActionEvent e) {
                    m_mainPanel.findPrev();
                }
            });
            menuItemFindPrev.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F3, InputEvent.SHIFT_DOWN_MASK));
            menuSearch.add(menuItemFindPrev);

            bar.add(menuSearch);
        }

        {
            JMenu menuHelp = new JMenu("Help");
            menuHelp.setIcon(ICON_MENU_HELP);

            final JMenuItem menuItemHelpGuide = new JMenuItem();
            menuItemHelpGuide.setAction(new AbstractAction("Help Guide", ICON_MENU_GUIDE) {

                public void actionPerformed(ActionEvent e) {
                    HTMLViewer panelHtml = new HTMLViewer();
                    panelHtml.setPreferredSize(new Dimension(950, 692));
                    panelHtml.setPage(UtilsIO.getResourceURL("/resources/helpHtml/helpGuide.html"));
                    JFrame dialogFrameHelpGuide = UtilsUI.getTemporaryDialogFrame(SyncFrame.this, panelHtml, "Help Guide", ICON_MENU_GUIDE.getImage(), false, true, false);
                    dialogFrameHelpGuide.addWindowListener(new WindowAdapter() {

                        @Override
                        public void windowClosing(WindowEvent evt) {
                            menuItemHelpGuide.setEnabled(true);
                        }
                    });
                    menuItemHelpGuide.setEnabled(false);
                    dialogFrameHelpGuide.setVisible(true);
                }
            });
            menuItemHelpGuide.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0));
            menuHelp.add(menuItemHelpGuide);

            JMenuItem menuItemAbout = new JMenuItem(new AbstractAction("About", ICON_MENU_ABOUT) {

                public void actionPerformed(ActionEvent e) {
                    HTMLViewer panelHtml = new HTMLViewer();
                    panelHtml.setPreferredSize(new Dimension(355, 124));
                    panelHtml.setPage(UtilsIO.getResourceURL("/resources/helpHtml/about.html"));
                    JDialog temp = UtilsUI.getTemporaryDialog(SyncFrame.this, panelHtml, "About: " + APP_TITLE + " " + APP_VERSION_STRING, ICON_MENU_ABOUT.getImage(), true, true, false);
                    temp.setResizable(false);
                    temp.setVisible(true);
                }
            });
            menuItemAbout.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, InputEvent.CTRL_DOWN_MASK));
            menuHelp.add(menuItemAbout);

            bar.add(menuHelp);
        }

        return bar;
    }

    private void showSynchronizationSettings() {
        JDialog dialog = UtilsUI.getTemporaryDialog(SyncFrame.this, m_panelConfiguration, "Synchronization Settings", ICON_MENU_SETTINGS.getImage(), true, true, false);
        dialog.setResizable(false);
        dialog.setVisible(true);
    }

    private boolean passWarnForOpenNewFile() {
        if (SHOW_WARNING_ON_OPEN_NEW_FILE && m_mainPanel.areAnySubsLoaded() && !UtilsUI.showConfirmMessage(this, "Open new subtitle file?")) {
            return false;
        }
        return true;
    }

    private boolean openSubtitleFile(File subtitleFile) {
        SubtitleType subFileType = getSubtitleTypeFromFileName(subtitleFile.getName());
        boolean returnValue = false, ioException = false;
        if (subFileType == null) {
            UtilsUI.showErrorMessage(this, "File type not supported.");
        } else {
            try {
                ArrayList<SubtitleBlock> readBlocks = Subsync.readAllBlocks(subtitleFile, subFileType);
                if (readBlocks.isEmpty()) {
                    UtilsUI.showErrorMessage(this, "File contains no subtitles");
                } else {
                    m_mainPanel.resetWithConfirmation(readBlocks);
                    m_menuItemExport.setEnabled(true);

                    setTitle(APP_TITLE_LONG + " - " + subtitleFile.getName());
                    returnValue = true;
                }
            } catch (FileParseException exc) {
                UtilsUI.showErrorMessage(this, "Error at line " + exc.getErrorLineNumber() + ": \n\"" + exc.getErrorLine() + "\"\n(" + exc.getMessage() + ")");
                //LoggingUtils.log(SyncPanel.class, exc);
            } catch (IOException ioex) {
                UtilsUI.showErrorMessage(this, "Error reading file.");
                ioException = true;
                //LoggingUtils.log(SyncPanel.class, ioex);
            } catch (Exception e) {
                UtilsLogging.log(SyncPanel.class, e);
            }
        }

        if (returnValue || ioException) {
            m_fileLocationCurrent = subtitleFile;
        } else {
            m_fileLocationCurrent = subtitleFile.getParentFile();
        }
        return returnValue;
    }

    private boolean saveSubtitleFile(File subtitleFile) {
        boolean returnValue = false, ioException = false;
        SubtitleType subFileType = getSubtitleTypeFromFileName(subtitleFile.getName());
        if (subFileType == null) {
            UtilsUI.showErrorMessage(this, "Extension file type not supported.");
        } else {
            try {
                ArrayList<SubtitleBlock> subtitles;
                try {
                    if (subFileType == SubtitleType.SRT_TYPE) {
                        subtitles = m_mainPanel.getSubsSynchronizedAsSRT((Integer) m_spinnerSRTStartingIndex.getValue());
                    } else {
                        subtitles = m_mainPanel.getSubsSynchronizedAsSUB((Double) m_spinnerSUBFps.getValue());
                    }
                } catch (WarningException we) {
                    subtitles = (ArrayList<SubtitleBlock>) we.getReturnValue();
                    UtilsUI.showWarningMessage(this, we.getMessage());
                }
                Subsync.writeAllBlocks(subtitleFile, subtitles);
                returnValue = true;
            } catch (InvalidParameterValuesException iae) {
                UtilsUI.showErrorMessage(this, iae.getMessage());
            } catch (IOException ioex) {
                UtilsUI.showErrorMessage(this, "Error writing file.");
                ioException = true;
                //LoggingUtils.log(SyncPanel.class, ioex);
            } catch (Exception e) {
                UtilsLogging.log(SyncPanel.class, e);
            }
        }
        if (returnValue || ioException) {
            m_fileLocationCurrent = subtitleFile;
        } else {
            m_fileLocationCurrent = subtitleFile.getParentFile();
        }
        return returnValue;
    }

    private SubtitleType getSubtitleTypeFromFileName(String filename) {
        String extensionLowCase = UtilsFiles.getExtension(filename).toLowerCase();
        for (SubtitleType subtitleType : SubtitleType.values()) {
            if (subtitleType.getLowerCaseExtension().equals(extensionLowCase)) {
                return subtitleType;
            }
        }
        return null;
    }
}
