/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package gui;

import core.SUBBlock;
import core.SubtitleType;
import core.Subsync;
import core.SubtitleBlock;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JToolBar;
import utils.UtilsUI;
import utils.UtilsImages;
import utils.UtilsLookAndFeel;
import utils.exceptions.InvalidParameterValuesException;
import utils.exceptions.WarningException;

/**
 *
 * Date: 5 Μαρ 2012
 * @author Jason Koukas
 */
public class SyncPanel extends JPanel {

    private static final Icon ICON_SUBTITLE_STARTING = UtilsImages.loadImageIcon("/resources/icons/startingSubtitle-16.png");
    private static final Icon ICON_SUBTITLE_ENDING = UtilsImages.loadImageIcon("/resources/icons/endingSubtitle-16.png");
    private static final Icon ICON_SUBTITLE_FOUND = UtilsImages.loadImageIcon("/resources/icons/foundSubtitle-16.png");
    protected static final Color COLOR_SELECT = new Color(0xFFEEBB); //The Color of the selected found subtitle
    protected static final Color COLOR_STARTING_SUBTITLE = new Color(0xBBFFBB); //The Color of the selected starting subtitle
    protected static final Color COLOR_ENDING_SUBTITLE = new Color(0xFFBBBB); //The Color of the selected ending subtitle
    //
    protected SubtitlesPane m_subtitlesPane;
    protected JToolBar m_toolBar;
    protected SearchPanel m_searchPanel;
    protected SubtitleDisplayArea m_foundSubtitle;
    //
    protected JButton m_scrollToStartSub;
    protected JButton m_scrollToEndSub;
    protected JButton m_scrollToFoundSub;
    //
    protected SubtitleDisplayArea m_startingSubtitleArea;
    protected SubtitleDisplayArea m_endingSubtitleArea;
    //
    protected TimestampSRTPanel m_panelStart;
    protected TimestampSRTPanel m_panelEnd;
    protected double m_videoFramerate;
    //
    protected MouseListener m_mouseListener;
    protected JMenuItem m_itemClearStartingSubtitle;
    protected JMenuItem m_itemClearEndingSubtitle;

    public SyncPanel() {
        setLayout(new BorderLayout(4, 4));

        m_subtitlesPane = new SubtitlesPane();
        m_mouseListener = mouseListenerCreate();
        m_searchPanel = new SearchPanel(this);
        m_videoFramerate = SUBBlock.POPULAR_FRAMERATES[0];

        m_itemClearStartingSubtitle = new JMenuItem(new AbstractAction("Clear starting subtitle", ICON_SUBTITLE_STARTING) {

            public void actionPerformed(ActionEvent e) {
                clearStart();
                m_panelStart.reset();
            }
        });

        m_itemClearEndingSubtitle = new JMenuItem(new AbstractAction("Clear ending subtitle", ICON_SUBTITLE_ENDING) {

            public void actionPerformed(ActionEvent e) {
                clearEnd();
                m_panelEnd.reset();
            }
        });

        JPanel panelTop = new JPanel(new BorderLayout());
        {
            m_toolBar = UtilsUI.getToolBar();

            m_scrollToStartSub = new JButton(new AbstractAction("", ICON_SUBTITLE_STARTING) {

                public void actionPerformed(ActionEvent e) {
                    if (m_startingSubtitleArea != null) {
                        m_subtitlesPane.scrollToBlock(m_startingSubtitleArea);
                    }
                }
            });
            m_scrollToStartSub.setToolTipText("Scroll to starting subtitle");
            m_scrollToStartSub.setEnabled(false);

            m_scrollToEndSub = new JButton(new AbstractAction("", ICON_SUBTITLE_ENDING) {

                public void actionPerformed(ActionEvent e) {
                    if (m_endingSubtitleArea != null) {
                        m_subtitlesPane.scrollToBlock(m_endingSubtitleArea);
                    }
                }
            });
            m_scrollToEndSub.setToolTipText("Scroll to ending subtitle");
            m_scrollToEndSub.setEnabled(false);

            m_scrollToFoundSub = new JButton(new AbstractAction("", ICON_SUBTITLE_FOUND) {

                public void actionPerformed(ActionEvent e) {
                    if (m_foundSubtitle != null) {
                        m_subtitlesPane.scrollToBlock(m_foundSubtitle);
                    }
                }
            });
            m_scrollToFoundSub.setToolTipText("Scroll to found subtitle");
            m_scrollToFoundSub.setEnabled(false);

            m_toolBar.add(m_scrollToStartSub);
            m_toolBar.add(m_scrollToEndSub);
            m_toolBar.add(m_scrollToFoundSub);


            panelTop.add(m_toolBar, BorderLayout.NORTH);
            panelTop.add(m_searchPanel, BorderLayout.SOUTH);
        }

        JPanel panelTimestamps = new JPanel();
        {
            panelTimestamps.setLayout(new BorderLayout(5, 0));
            Dimension timestampSRTSize = new Dimension(240, 85);

            m_panelStart = new TimestampSRTPanel(0, 0, 0, 0);
            JPanel panelStartWrapper = UtilsUI.getWrapperPanel(m_panelStart, false);
            panelStartWrapper.setPreferredSize(timestampSRTSize);
            panelStartWrapper.setBorder(UtilsLookAndFeel.getTitledBorder("Starting subtitle observed timestamp"));
            m_panelStart.extendCompleteDnDBehaviourTo(panelStartWrapper);

            m_panelEnd = new TimestampSRTPanel(0, 0, 0, 0);
            JPanel panelEndWrapper = UtilsUI.getWrapperPanel(m_panelEnd, false);
            panelEndWrapper.setPreferredSize(timestampSRTSize);
            panelEndWrapper.setBorder(UtilsLookAndFeel.getTitledBorder("Ending subtitle observed timestamp"));
            m_panelEnd.extendCompleteDnDBehaviourTo(panelEndWrapper);

            panelTimestamps.add(BorderLayout.WEST, panelStartWrapper);
            panelTimestamps.add(BorderLayout.EAST, panelEndWrapper);
            panelTimestamps = UtilsUI.getWrapperPanel(panelTimestamps, false);
        }

        add(panelTop, BorderLayout.NORTH);
        add(m_subtitlesPane.getScrollPaneContainer(), BorderLayout.CENTER);
        add(panelTimestamps, BorderLayout.SOUTH);

        reset(new ArrayList<SubtitleBlock>());
    }

    public final void resetWithConfirmation(ArrayList<SubtitleBlock> listOfSubs) {
        reset(listOfSubs, (m_panelStart.getTimestamp() != 0 || m_panelEnd.getTimestamp() != 0)
                && UtilsUI.showConfirmMessage(this, "Clear observed timestamps?"));
    }

    public final void reset(ArrayList<SubtitleBlock> listOfSubs) {
        reset(listOfSubs, true);
    }

    public final void reset(ArrayList<SubtitleBlock> listOfSubs, boolean clearObservedTimestamps) {
        m_subtitlesPane.reset(listOfSubs);
        m_searchPanel.reset();

        if (clearObservedTimestamps) {
            m_panelStart.reset();
            m_panelEnd.reset();
        }

        clearStart();
        clearEnd();
        clearFoundSubtitle();

        ArrayList<SubtitleDisplayArea> subtitleDisplayAreas = m_subtitlesPane.getSubtitles();
        for (SubtitleDisplayArea subtitleArea : subtitleDisplayAreas) {
            subtitleArea.addMouseListener(m_mouseListener);
        }
    }

    public SubtitlesPane getSubtitlesPane() {
        return m_subtitlesPane;
    }

    public JToolBar getToolBar() {
        return m_toolBar;
    }

    public ArrayList<SubtitleBlock> getSubsSynchronizedAsSRT(int indexStarting) throws InvalidParameterValuesException, WarningException {
        if (indexStarting < 0) {
            throw new InvalidParameterValuesException("Starting index shouldn't be negative.");
        }
        return getSubtitlesSynchronized(SubtitleType.SRT_TYPE, indexStarting);
    }

    public ArrayList<SubtitleBlock> getSubsSynchronizedAsSUB(double fps) throws InvalidParameterValuesException, WarningException {
        if (fps < SUBBlock.FPS_MIN || fps > SUBBlock.FPS_MAX) {
            throw new InvalidParameterValuesException("Framerate should be between " + SUBBlock.FPS_MIN + " and " + SUBBlock.FPS_MAX + ".");
        }
        return getSubtitlesSynchronized(SubtitleType.SUB_TYPE, fps);
    }

    public boolean areAnySubsLoaded() {
        return !m_subtitlesPane.getSubtitles().isEmpty();
    }

    public void verifySynchronizationValuesValid() throws InvalidParameterValuesException {
        if (!areAnySubsLoaded()) {
            throw new InvalidParameterValuesException("No subtitles are loaded.");
        }
        if (m_startingSubtitleArea == null || m_endingSubtitleArea == null) {
            throw new InvalidParameterValuesException("Both starting and ending subtitles must be set.");
        }
        int selectedStart = m_startingSubtitleArea.getSubtitleBlock().getStart(), selectedEnd = m_endingSubtitleArea.getSubtitleBlock().getStart();
        if (selectedStart == selectedEnd) {
            throw new InvalidParameterValuesException("Selected subtitles shouldn't have the same timestamp.");
        }
    }

    public void updateVideoFramerate(double framerate) {
        //An thelame, edw tha kaname update ta timestamps kai ta values twn sub-files
        m_videoFramerate = framerate;
    }

    public void findNext() {
        requestFocus();
        m_searchPanel.findNext();
    }

    public void findPrev() {
        requestFocus();
        m_searchPanel.findPrev();
    }

    public void selectFoundSubtitle(int areaIndex) {
        SubtitleDisplayArea displayArea = m_subtitlesPane.getSubtitles().get(areaIndex);
        if (displayArea != m_foundSubtitle) {
            clearFoundSubtitle();
            m_foundSubtitle = displayArea;
            updateAreaColor(m_foundSubtitle);
            m_scrollToFoundSub.setEnabled(true);
        }
        getSubtitlesPane().scrollToBlock(displayArea);
    }

    public void clearFoundSubtitle() {
        if (m_foundSubtitle != null) {
            SubtitleDisplayArea found = m_foundSubtitle;
            m_foundSubtitle = null;
            updateAreaColor(found);
            m_scrollToFoundSub.setEnabled(false);
        }
    }

    @Override
    public void requestFocus() {
        m_searchPanel.requestFocus();
    }

    /**
     * Private Helpful Methods
     */
    private ArrayList<SubtitleBlock> getSubtitlesSynchronized(SubtitleType outputFormatType, double destFpsOrFirstIndex) throws InvalidParameterValuesException, WarningException {
        verifySynchronizationValuesValid();

        ArrayList<SubtitleBlock> blocks = new ArrayList<SubtitleBlock>();
        for (SubtitleDisplayArea displayArea : m_subtitlesPane.getSubtitles()) {
            blocks.add(displayArea.getSubtitleBlock());
        }

        return Subsync.synchronize(blocks, m_startingSubtitleArea.getSubtitleBlock().getType(), outputFormatType,
                m_startingSubtitleArea.getSubtitleBlock().getStart(), m_endingSubtitleArea.getSubtitleBlock().getStart(),
                m_panelStart.getTimestamp(), m_panelEnd.getTimestamp(), destFpsOrFirstIndex);
    }

    private void updateAreaColor(SubtitleDisplayArea displayArea) {
        Color variantBackground;

        if (displayArea == m_foundSubtitle) {
            if (displayArea == m_startingSubtitleArea) {
                variantBackground = COLOR_STARTING_SUBTITLE;
            } else if (displayArea == m_endingSubtitleArea) {
                variantBackground = COLOR_ENDING_SUBTITLE;
            } else {
                variantBackground = SubtitleDisplayArea.COLOR_TIMESTAMP_BACKGROUND;
            }
            displayArea.setTimestampBackground(variantBackground);
            displayArea.setBackground(COLOR_SELECT);
        } else {
            if (displayArea == m_startingSubtitleArea) {
                variantBackground = COLOR_STARTING_SUBTITLE;
            } else if (displayArea == m_endingSubtitleArea) {
                variantBackground = COLOR_ENDING_SUBTITLE;
            } else {
                variantBackground = SubtitleDisplayArea.COLOR_DEFAULT_BACKGROUND;
            }
            displayArea.setTimestampBackground(SubtitleDisplayArea.COLOR_TIMESTAMP_BACKGROUND);
            displayArea.setBackground(variantBackground);
        }
    }

    private void selectStart(SubtitleDisplayArea displayArea) {
        if (m_startingSubtitleArea != displayArea) {
            clearStart();
            if (displayArea == m_endingSubtitleArea) {
                clearEnd();
            }
            m_startingSubtitleArea = displayArea;
            updateAreaColor(displayArea);
            m_scrollToStartSub.setEnabled(true);
        }
        if (m_panelStart.getTimestamp() == 0) {
            copyTimestamp(displayArea, m_panelStart);
        }
    }

    private void clearStart() {
        if (m_startingSubtitleArea != null) {
            SubtitleDisplayArea displayArea = m_startingSubtitleArea;
            m_startingSubtitleArea = null;
            updateAreaColor(displayArea);
            m_scrollToStartSub.setEnabled(false);
        }
    }

    private void selectEnd(SubtitleDisplayArea displayArea) {
        if (m_endingSubtitleArea != displayArea) {
            clearEnd();
            if (displayArea == m_startingSubtitleArea) {
                clearStart();
            }
            m_endingSubtitleArea = displayArea;
            updateAreaColor(displayArea);
            m_scrollToEndSub.setEnabled(true);
        }
        if (m_panelEnd.getTimestamp() == 0) {
            copyTimestamp(displayArea, m_panelEnd);
        }
    }

    private void clearEnd() {
        if (m_endingSubtitleArea != null) {
            SubtitleDisplayArea displayArea = m_endingSubtitleArea;
            m_endingSubtitleArea = null;
            updateAreaColor(displayArea);
            m_scrollToEndSub.setEnabled(false);
        }
    }

    private void copyTimestamp(SubtitleDisplayArea displayArea, TimestampSRTPanel timestampPanel) {
        SubtitleBlock block = displayArea.getSubtitleBlock();
        if (block.getType() == SubtitleType.SRT_TYPE) {
            timestampPanel.setTimestamp(block.getStart());
        } else {
            timestampPanel.setTimestamp((int) Subsync.frameToMilliseconds(block.getStart(), m_videoFramerate));
        }
    }

    private MouseListener mouseListenerCreate() {
        return new MouseAdapter() {

            @Override
            public void mousePressed(MouseEvent mevt) {
                showPopupIfPopupTrigger(mevt);
            }

            @Override
            public void mouseReleased(MouseEvent mevt) {
                if (!showPopupIfPopupTrigger(mevt)) {
                    if (mevt.getClickCount() == 1) {
                        m_searchPanel.setSelectedDisplayArea((SubtitleDisplayArea) mevt.getSource());
                    }
                }
            }
        };
    }

    private boolean showPopupIfPopupTrigger(MouseEvent mevt) {
        if (mevt.isPopupTrigger()) {
            JPopupMenu popup = new JPopupMenu();

            final SubtitleDisplayArea sourceArea = (SubtitleDisplayArea) mevt.getSource();
            ArrayList<SubtitleDisplayArea> subAreas = m_subtitlesPane.getSubtitles();
            int subtitlesDiv2 = subAreas.size() >> 1;
            int firstSubtitleIndex = subAreas.indexOf(m_startingSubtitleArea);
            int lastSubtitleIndex = subAreas.indexOf(m_endingSubtitleArea);
            int sourceSubtitleIndex = subAreas.indexOf(sourceArea);

            if (sourceSubtitleIndex < subtitlesDiv2) {
                if (sourceArea == m_startingSubtitleArea) {
                    popup.add(m_itemClearStartingSubtitle);
                } else if (lastSubtitleIndex < 0 || sourceSubtitleIndex < lastSubtitleIndex) {
                    popup.add(new AbstractAction("Set as starting subtitle", ICON_SUBTITLE_STARTING) {

                        public void actionPerformed(ActionEvent e) {
                            selectStart(sourceArea);
                        }
                    });
                }
                popup.add(new AbstractAction("Copy to observed starting timestamp", ICON_SUBTITLE_STARTING) {

                    public void actionPerformed(ActionEvent e) {
                        copyTimestamp(sourceArea, m_panelStart);
                    }
                });
            } else { //if (sourceSubtitleIndex >= subtitlesDiv2) {
                if (sourceArea == m_endingSubtitleArea) {
                    popup.add(m_itemClearEndingSubtitle);
                } else if (firstSubtitleIndex < 0 || sourceSubtitleIndex > firstSubtitleIndex) {
                    popup.add(new AbstractAction("Set as ending subtitle", ICON_SUBTITLE_ENDING) {

                        public void actionPerformed(ActionEvent e) {
                            selectEnd(sourceArea);
                        }
                    });
                }
                popup.add(new AbstractAction("Copy to observed ending timestamp", ICON_SUBTITLE_ENDING) {

                    public void actionPerformed(ActionEvent e) {
                        copyTimestamp(sourceArea, m_panelEnd);
                    }
                });
            }

            popup.show(mevt.getComponent(), mevt.getX(), mevt.getY());
            return true;
        }
        return false;
    }
}
