/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package gui;

import core.TimeConstants;
import core.SRTBlock;
import java.awt.BorderLayout;
import java.awt.Component;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerModel;
import javax.swing.event.ChangeListener;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.MouseMotionListener;
import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import utils.UtilsUI;
import utils.UtilsIO;
import utils.UtilsLogging;
import utils.UtilsImages;
import utils.UtilsLookAndFeel;
import utils.dnd.TransferHandlerCustomExtend;
import utils.dnd.TransferableGeneric;
import utils.exceptions.SyntaxException;
import utils.ui.JSpinnerEditable;
import utils.ui.JSpinnerEditable.SpinnerModelInt;

/**
 *
 * @author ikoukas
 */
public class TimestampSRTPanel extends JPanel {

    private static final Icon ICON_COPY = UtilsImages.loadImageIcon("/resources/icons/copy-16.png");
    private static final Icon ICON_PASTE = UtilsImages.loadImageIcon("/resources/icons/paste-16.png");
    private static final Icon ICON_CLEAR = UtilsImages.loadImageIcon("/resources/icons/clear-16.png");
    private static final DataFlavor[] SUPPORTED_FLAVORS_STRING = new DataFlavor[]{DataFlavor.stringFlavor};
    private static final Color FIELD_BACKGROUND = Color.white;
    private JSpinner m_hours;
    private JSpinner m_minutes;
    private JSpinner m_seconds;
    private JSpinner m_millis;

    private class TransferHandlerCopyPaste extends TransferHandlerCustomExtend {

        public TransferHandlerCopyPaste(JComponent overridenComponent, boolean validateTransferData) {
            super(overridenComponent, SUPPORTED_FLAVORS_STRING, validateTransferData); //To validate exei na kanei me to an sto drop tha fainetai
            //oti epitrepetai i oxi.. Sta textFields, theloume na fainetai oti epitrepetai, panta, giati mporei na theloume na kanoume paste, apla..
        }

        /**
         * Methods Overriden from TransferHandler (for drag)
         */
        @Override
        public Transferable createTransferable(JComponent c) {
            return new TransferableGeneric<String>(SRTBlock.millisecondsToTime(getTimestamp()), SUPPORTED_FLAVORS_STRING);
        }

        @Override
        public int getSourceActions(JComponent c) {
            return DnDConstants.ACTION_COPY_OR_MOVE | super.getSourceActions(c);
        }

        /**
         * Methods Overriden from TransferHandler (for drop)
         */
        @Override
        public boolean validateTransferData(Transferable transferable, Object transferData, DataFlavor flavorPreferred) {
            String value = (String) transferData;  //....
            try {
                SRTBlock.timeToMilliseconds(value);
            } catch (SyntaxException ex) {
                //LoggingUtils.log(getClass(), ex);
                return false;
            }
            return true;
        }

        @Override
        public boolean importData(Component comp, Transferable transferable, Object transferData, DataFlavor usedFlavor) {
            String value = (String) transferData;
            try {
                setTimestamp(SRTBlock.timeToMilliseconds(value));
            } catch (SyntaxException ex) { //Won't happen (?..)
                //LoggingUtils.log(getClass(), ex);
                return m_replacedHandler.importData((JComponent) comp, transferable);
            }
            return true;
        }
    }

    public TimestampSRTPanel(int hours, int minutes, int seconds, int milliSeconds) {
        super(new GridLayout(2, 1));
        JLabel label = new JLabel("hrs : mins : secs , msecs");
        label.setFont(UtilsLookAndFeel.FONT_TITLES);
        label.setHorizontalAlignment(JLabel.CENTER);

        add(label);

        JPanel panelContents;
        {
            panelContents = new JPanel(new GridLayout(1, 4));

            m_hours = createSpinner(0, SRTBlock.HOURS_MAX_VALUE, hours);
            m_minutes = createSpinner(0, TimeConstants.MINUTES_SECONDS_PER_HOUR_MINUTE - 1, minutes);
            m_minutes.addChangeListener(createChangeListener(minutes, m_hours));
            m_seconds = createSpinner(0, TimeConstants.MINUTES_SECONDS_PER_HOUR_MINUTE - 1, seconds);
            m_seconds.addChangeListener(createChangeListener(seconds, m_minutes));
            m_millis = createSpinner(0, TimeConstants.MILLIS_PER_SECOND - 1, milliSeconds);
            m_millis.addChangeListener(createChangeListener(milliSeconds, m_seconds));

            panelContents.add(m_hours);
            panelContents.add(m_minutes);
            panelContents.add(m_seconds);
            panelContents.add(m_millis);
        }

        add(panelContents);

        new TransferHandlerCopyPaste(this, true);

        Dimension dim = new Dimension(180, 45);
        setMinimumSize(dim);
        setPreferredSize(dim);
    }

    public int getTimestamp() {
        return SRTBlock.timeValuesToMilliseconds((Integer) m_hours.getValue(),
                (Integer) m_minutes.getValue(), (Integer) m_seconds.getValue(),
                (Integer) m_millis.getValue());
    }

    public void setTimestamp(int milliseconds) {
        int[] seperatedVals = SRTBlock.millisecondsToSeperateTimeValues(milliseconds);
        m_millis.setValue(seperatedVals[3]);
        m_seconds.setValue(seperatedVals[2]);
        m_minutes.setValue(seperatedVals[1]);
        m_hours.setValue(seperatedVals[0]);
        //.. anapoda wste na min exoume provlima me tin allagi timwn tou epomenou unit..
    }

    public void reset() {
        setTimestamp(0);
    }

    /**
     * Methods concerning drag/drop functions
     */
    public MouseListener getCopyPasteMouseListener() {
        return new MouseAdapter() {

            @Override
            public void mousePressed(MouseEvent mevt) {
                showPopupIfPopupTrigger(mevt);
            }

            @Override
            public void mouseReleased(MouseEvent mevt) {
                showPopupIfPopupTrigger(mevt);
            }
        };
    }

    public MouseMotionListener getDragGestureMouseMotionListener() {
        return new MouseMotionAdapter() {

            @Override
            public void mouseDragged(MouseEvent e) {
                getTransferHandler().exportAsDrag(TimestampSRTPanel.this, e, DnDConstants.ACTION_COPY);
            }
        };
    }

    public void extendTransferHandlerBehaviourTo(JComponent comp) {
        new TransferHandlerCopyPaste(comp, true);
    }

    public void extendCompleteDnDBehaviourTo(JComponent comp) {
        comp.addMouseListener(getCopyPasteMouseListener());
        comp.addMouseMotionListener(getDragGestureMouseMotionListener());
        extendTransferHandlerBehaviourTo(comp);
    }

    /**
     * Private, Helpful Methods
     */
    private JSpinner createSpinner(int min, int max, int value) {
        JSpinnerEditable spinner = new JSpinnerEditable(new SpinnerModelInt(min, max, value));
        JFormattedTextField valueField = spinner.getFormattedTextField();
        valueField.setBackground(FIELD_BACKGROUND);
        new TransferHandlerCopyPaste(valueField, false);
        return spinner;
    }

    private ChangeListener createChangeListener(final int currentVal, final JSpinner prev) {
        return new ChangeListener() {

            private int t_prevVal = currentVal;

            public void stateChanged(ChangeEvent e) {
                SpinnerModelInt model = (SpinnerModelInt) ((JSpinner) e.getSource()).getModel();
                int currentValue = (Integer) model.getValue();

                if (t_prevVal == model.getMaxValue() && currentValue == model.getMinValue()) {
                    SpinnerModel modelPrev = prev.getModel();
                    modelPrev.setValue(modelPrev.getNextValue());
                } else if (t_prevVal == model.getMinValue() && currentValue == model.getMaxValue()) {
                    SpinnerModel modelPrev = prev.getModel();
                    modelPrev.setValue(modelPrev.getPreviousValue());
                }

                t_prevVal = currentValue;
            }
        };
    }

    private boolean showPopupIfPopupTrigger(MouseEvent mevt) {
        if (mevt.isPopupTrigger()) {
            JPopupMenu popup = new JPopupMenu();

            popup.add(new AbstractAction("Copy", ICON_COPY) {

                public void actionPerformed(ActionEvent e) {
                    getTransferHandler().exportToClipboard(TimestampSRTPanel.this, Toolkit.getDefaultToolkit().getSystemClipboard(), DnDConstants.ACTION_COPY);
                }
            });

            String stringValue = (String) UtilsIO.getSystemClipboard(SUPPORTED_FLAVORS_STRING[0]);
            if (stringValue != null) {
                if (SRTBlock.validateTimeString(stringValue)) {
                    popup.add(new AbstractAction("Paste", ICON_PASTE) {

                        public void actionPerformed(ActionEvent e) {
                            try {
                                setTimestamp(SRTBlock.timeToMilliseconds((String) UtilsIO.getSystemClipboard(SUPPORTED_FLAVORS_STRING[0])));
                            } catch (SyntaxException ex) {
                                //shouldn't happen
                            }
                        }
                    });
                }
            }

            popup.addSeparator();
            popup.add(new AbstractAction("Clear", ICON_CLEAR) {

                public void actionPerformed(ActionEvent e) {
                    reset();
                }
            });

            popup.show(mevt.getComponent(), mevt.getX(), mevt.getY());
            return true;
        }
        return false;
    }

    public static void main(String[] args) {
        // Set cross-platform Java L&F (also called "Metal")
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Exception e) {
            UtilsLogging.log(TimestampSRTPanel.class, e);
        }

        TimestampSRTPanel timestampPanel = new TimestampSRTPanel(0, 0, 0, 0);
        JTextPane textPane = new JTextPane();
        textPane.setText("232:23:45,666\n1232:23:45,666");

        JPanel mainPanel = new JPanel(new BorderLayout());
        mainPanel.add(BorderLayout.NORTH, timestampPanel);
        mainPanel.add(BorderLayout.CENTER, new JScrollPane(textPane));

        JFrame parentFrame = new JFrame();
        parentFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        parentFrame.getContentPane().add(mainPanel);
        parentFrame.setSize(640, 480);
        UtilsUI.centerComponentOnScreen(parentFrame);
        parentFrame.setVisible(true);
    }
}
