package utils;

import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.FileNotFoundException;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;
import utils.exceptions.InvalidParameterValuesException;
import utils.exceptions.WarningException;

/**
 * 18 Απρ 2010
 * 
 * @author Jason Koukas
 */
public class UtilsFiles {

    private static JFileChooser MAIN_FILE_CHOOSER;
    private static File FILE_SELECTED_PREVIOUS_VALUE;
    private static File[] FILES_CURRENT_OPEN_SAVE;

    public static File getOpenFileDialog(Component parent, FileNameExtensionFilter[] filters) {
        return getFileSelectDialogUntilSelectionValid(parent, filters, null, true);
    }

    public static File getOpenFileDialog(Component parent, FileNameExtensionFilter[] filters, File preSelectedFile) {
        return getFileSelectDialogUntilSelectionValid(parent, filters, preSelectedFile, true);
    }

    public static File getSaveFileDialog(Component parent, FileNameExtensionFilter[] filters) {
        return getFileSelectDialogUntilSelectionValid(parent, filters, null, false);
    }

    public static File getSaveFileDialog(Component parent, FileNameExtensionFilter[] filters, File preSelectedFile) {
        return getFileSelectDialogUntilSelectionValid(parent, filters, preSelectedFile, false);
    }

    public static File getFileSelectDialogUntilSelectionValid(Component parent, FileNameExtensionFilter[] filters, File preSelectedFile, boolean open) {
        do {
            try {
                return getFileSelectDialog(parent, filters, preSelectedFile, open);
            } catch (WarningException ex) {
                /**
                 * If file already exists, confirm
                 */
                if (UtilsUI.showConfirmMessage(parent, ex.getMessage())) {
                    return (File) ex.getReturnValue();
                }
            } catch (FileNotFoundException ex) {
                UtilsUI.showErrorMessage(parent, ex.getMessage());
            } catch (InvalidParameterValuesException ex) {
                UtilsUI.showErrorMessage(parent, ex.getMessage());
            }
            //if not first time we try
            if (preSelectedFile != null) {
                preSelectedFile = null;
            }
        } while (true);
    }

    public static File getFileSelectDialog(Component parent, FileNameExtensionFilter[] filters, File preSelectedFileOrDirectory, boolean open)
            throws FileNotFoundException, InvalidParameterValuesException, WarningException {
        JFileChooser fileChooser = getFileChooser();
        if (preSelectedFileOrDirectory == null) {
            setChooserFile(fileChooser, getFileCurrent(open));
        } else {
            if (preSelectedFileOrDirectory.exists()) {
                setChooserFile(fileChooser, preSelectedFileOrDirectory);
            } else { //doesn't exist..
                preSelectedFileOrDirectory = null; //for safety of usage
                setChooserFile(fileChooser, getFileCurrent(open));
            }
        }

        int ret;
        FileFilter filter;
        {
            for (FileNameExtensionFilter tempFilter : filters) {
                fileChooser.addChoosableFileFilter(tempFilter);
            }

            {
                File selected = MAIN_FILE_CHOOSER.getSelectedFile();
                boolean filterSet = false;
                if (selected != null) {
                    String currentExtension = getExtension(selected.getName());

                    for (FileNameExtensionFilter tempFilter : filters) {
                        if (tempFilter.getExtensions().length == 1 && tempFilter.getExtensions()[0].equals(currentExtension)) {
                            fileChooser.setFileFilter(tempFilter);
                            filterSet = true;
                            break;
                        }
                    }
                }
                if (!filterSet) {
                    fileChooser.setFileFilter(filters[0]);
                }
            }

            if (open) {
                ret = fileChooser.showOpenDialog(parent);
            } else {
                ret = fileChooser.showSaveDialog(parent);
            }
            filter = fileChooser.getFileFilter();

            for (FileNameExtensionFilter tempFilter : filters) {
                fileChooser.removeChoosableFileFilter(tempFilter);
            }
        }

        if (ret == JFileChooser.APPROVE_OPTION) {
            File selected = MAIN_FILE_CHOOSER.getSelectedFile();
            /**
             * Validate extension and being a file
             */
            if ("".equals(getExtension(selected.getName())) && filter instanceof FileNameExtensionFilter
                    && ((FileNameExtensionFilter) filter).getExtensions().length == 1) {
                selected = new File(selected.getAbsolutePath() + "." + ((FileNameExtensionFilter) filter).getExtensions()[0]);
            } else {
                boolean accepted = false;
                for (FileFilter tempFilter : filters) {
                    if (tempFilter.accept(selected)) {
                        accepted = true;
                        break;
                    }
                }

                if (!accepted) { //Pithanws to isDirectory() de borei na epistrepsei true..
                    trySelectParentFile(selected, open);
                    throw new InvalidParameterValuesException("File type not allowed: ." + getExtension(selected.getName())
                            + "\nonly " + getAcceptedExtensionsString(((FileNameExtensionFilter) filter).getExtensions()) + " extensions are valid.");
                }
            }


            /**
             * Make sure opened file exists
             */
            if (open) {
                if (!selected.exists()) {
                    trySelectParentFile(selected, open);
                    throw new FileNotFoundException("File not found.");
                }
            } else if (selected.exists()) {
                setFileCurrent(selected, open);
                throw new WarningException(selected, "File already exists, overwrite?");
            } else if (!selected.getParentFile().exists()) {
                throw new FileNotFoundException("Invalid path.");
            }
            setFileCurrent(selected, open);
            return selected;
        } else {
            return null;
        }
    }

    public static FileNameExtensionFilter createFileNameExtensionFilter(String filterNamePlain, String[] extensions) {
        String filterName = filterNamePlain + " " + getAcceptedExtensionsString(extensions);
        return new FileNameExtensionFilter(filterName, extensions);
    }

    public static String getAcceptedExtensionsString(String[] extensions) {
        String acceptedExtensionsString = "[";
        int i = 0;
        for (String extension : extensions) {
            extension = extension.toLowerCase();
            if (i == 0) {
                acceptedExtensionsString += "." + extension;
            } else {
                acceptedExtensionsString += ", ." + extension;
            }
            i++;
        }
        acceptedExtensionsString += "]";
        return acceptedExtensionsString;
    }

    //if a file is like "abc" or "abc." the extension is ""
    public static String getExtension(String filename) {
        int i = filename.length() - 1;
        while (i >= 0 && filename.charAt(i) != '.') {
            i--;
        }
        if (i == -1 || i == filename.length() - 1) {
            return "";
        } else {
            return filename.substring(i + 1);
        }
    }

    //Periptwseis arxeiwn:
    //.test - return filename.substring(0, 0) - ara return "" i null
    //test - return filename
    //test. - return test
    //test.test - return test
    //
    //an "", epistrefei "", an "abc." epistrefei "abc"
    public static String removeExtension(String filename) {
        int i = filename.length() - 1;
        while (i >= 0 && filename.charAt(i) != '.') {
            i--;
        }
        if (i == -1)//has no dot
        {
            return filename;
        } //peritto
        //else if(i == filename.length()-1)
        //{
        //	return filename.substring(0, i);
        //}
        else {
            return filename.substring(0, i);
        }
    }

    public static String changeExtension(String filename, String newExtension) {
        return removeExtension(filename) + "." + newExtension;
    }

    public static File getFileCurrent(boolean open) {
        if (open) {
            return getFileCurrent(0, 1);
        } else {
            return getFileCurrent(1, 0);
        }
    }

    private static void setChooserFile(JFileChooser fileChooser, File fileCurrent) {
        if (fileCurrent.isDirectory()) {
            fileChooser.setCurrentDirectory(fileCurrent);
            fileChooser.setSelectedFile(null);
        } else {
            fileChooser.setSelectedFile(fileCurrent); //if it's directory it's ok..
        }
    }

    private static void trySelectParentFile(File selected, boolean open) {
        File parentFile = selected.getParentFile();
        if (parentFile.exists()) {
            setFileCurrent(parentFile, open);
        }
    }

    private static File getFileCurrent(int indexThis, int indexOther) {
        File fileOther = FILES_CURRENT_OPEN_SAVE[indexOther];
        File fileThis = FILES_CURRENT_OPEN_SAVE[indexThis];
        if (fileThis == null) {
            if (fileOther != null) {
                fileThis = fileOther;
            } else {
                fileThis = new File("./");
            }
            FILES_CURRENT_OPEN_SAVE[indexThis] = fileThis;
        }
        return fileThis;
    }

    private static void setFileCurrent(File fileCurrent, boolean open) {
        if (open) {
            FILES_CURRENT_OPEN_SAVE[0] = fileCurrent;
        } else {
            FILES_CURRENT_OPEN_SAVE[1] = fileCurrent;
        }
    }

    private static JFileChooser getFileChooser() {
        if (MAIN_FILE_CHOOSER == null) {
            MAIN_FILE_CHOOSER = new JFileChooser("./");
            MAIN_FILE_CHOOSER.setMultiSelectionEnabled(false);
            FILES_CURRENT_OPEN_SAVE = new File[2];

            MAIN_FILE_CHOOSER.addPropertyChangeListener(JFileChooser.SELECTED_FILE_CHANGED_PROPERTY, new PropertyChangeListener() {

                public void propertyChange(PropertyChangeEvent evt) {
                    FILE_SELECTED_PREVIOUS_VALUE = (File) evt.getOldValue();
                    //System.out.println(FILE_SELECTED_PREVIOUS_VALUE);
                }
            });

            MAIN_FILE_CHOOSER.addPropertyChangeListener(JFileChooser.FILE_FILTER_CHANGED_PROPERTY, new PropertyChangeListener() {

                public void propertyChange(PropertyChangeEvent evt) {
                    File selected = FILE_SELECTED_PREVIOUS_VALUE;
                    if (selected != null) {
                        Object filter = evt.getNewValue();
                        if (filter instanceof FileNameExtensionFilter && ((FileNameExtensionFilter) filter).getExtensions().length == 1) {
                            //System.out.println(((FileNameExtensionFilter) filter).getExtensions()[0]);
                            MAIN_FILE_CHOOSER.setSelectedFile(new File(removeExtension(selected.getAbsolutePath()) + "."
                                    + ((FileNameExtensionFilter) filter).getExtensions()[0]));
                        }
                    }
                }
            });
        }
        return MAIN_FILE_CHOOSER;
    }

    public static void main(String[] args) {
        JFrame frameTest = new JFrame();
        frameTest.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frameTest.setSize(420, 300);
        frameTest.setVisible(true);
        UtilsUI.centerComponentOnScreen(frameTest);
        getOpenFileDialog(frameTest, new FileNameExtensionFilter[]{
                    createFileNameExtensionFilter("SRT Files", new String[]{"srt"}),
                    createFileNameExtensionFilter("SUB Files", new String[]{"sub"})
                });
        getSaveFileDialog(frameTest, new FileNameExtensionFilter[]{
                    createFileNameExtensionFilter("SRT Files", new String[]{"srt"}),
                    createFileNameExtensionFilter("SUB Files", new String[]{"sub"})
                });
    }
}
