/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package utils;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.JTree;
import javax.swing.RootPaneContainer;

/**
 *
 * Date: 23 Φεβ 2012
 * @author Jason Koukas
 */
public class UtilsUI {

    public static final Class[] CLASSES_DEFAULT_TO_IGNORE_IN_SET_BACKGROUND_RECURSIVE = new Class[]{JTextField.class, JTextArea.class,
        JSpinner.class, JList.class, JComboBox.class,
        JTree.class, JTable.class};

    public static interface RecursiveBackgroundIgnore {
    };

    public static interface RecursiveBackgroundNoRecursion {
    };

    /**
     * Methods Regarding User Messages
     */
    public static boolean showConfirmMessage(Component component, String message) {
        return JOptionPane.showConfirmDialog(component, message, "Confirm", JOptionPane.OK_CANCEL_OPTION) == JOptionPane.YES_OPTION;
    }

    public static void showPlainMessage(Component component, String message) {
        JOptionPane.showMessageDialog(component, message, "Message", JOptionPane.PLAIN_MESSAGE);
    }

    public static void showInformationMessage(Component component, String message) {
        JOptionPane.showMessageDialog(component, message, "Information", JOptionPane.INFORMATION_MESSAGE);
    }

    public static void showWarningMessage(Component component, String message) {
        JOptionPane.showMessageDialog(component, message, "Warning", JOptionPane.WARNING_MESSAGE);
    }

    public static void showErrorMessage(Component component, String message) {
        JOptionPane.showMessageDialog(component, message, "Error", JOptionPane.ERROR_MESSAGE);
    }

    /**
     * Methods Regarding Centering of Components
     */
    public static void centerComponentOnScreen(Component component) {
        Dimension scrSize = Toolkit.getDefaultToolkit().getScreenSize();
        component.setLocation((scrSize.width - component.getWidth()) >> 1, (scrSize.height - component.getHeight()) >> 1);
    }

    public static void centerComponentOnComponentParent(Component component, Component parent) {
//        if (component.isShowing()) {
        Point parentScreenLocation = parent.getLocationOnScreen();
        int targetX = (int) (parentScreenLocation.getX() + ((parent.getWidth() - component.getWidth()) >> 1));
        int targetY = (int) (parentScreenLocation.getY() + ((parent.getHeight() - component.getHeight()) >> 1));

        Dimension scrSize = Toolkit.getDefaultToolkit().getScreenSize();
        if (targetX < 0) {
            targetX = 0;
        } else if (targetX + component.getWidth() > scrSize.width) {
            targetX = scrSize.width - component.getWidth();
        }
        if (targetY < 0) {
            targetY = 0;
        } else if (targetY + component.getHeight() > scrSize.height) {
            targetY = scrSize.height - component.getHeight();
        }
        component.setLocation(targetX, targetY);
//        } else {
//            centerComponentOnScreen(component);
//        }
    }

    public static void setBackgroundRecursive(Component comp, Color color) {
        setBackgroundRecursive(comp, color, CLASSES_DEFAULT_TO_IGNORE_IN_SET_BACKGROUND_RECURSIVE, null);
    }

    public static void setBackgroundRecursive(Component comp, Color color, Component[] instancesToIgnore) {
        setBackgroundRecursive(comp, color, CLASSES_DEFAULT_TO_IGNORE_IN_SET_BACKGROUND_RECURSIVE, instancesToIgnore);
    }

    public static void setBackgroundRecursive(Component comp, Color color, Class[] objectTypesToIgnore) {
        setBackgroundRecursive(comp, color, objectTypesToIgnore, null);
    }

    public static void setBackgroundRecursive(Component comp, Color color, Class[] objectClassesToIgnore, Component[] instancesToIgnore) {
        if (comp instanceof RecursiveBackgroundIgnore) {
            return;
        }
        if (objectClassesToIgnore != null) {
            for (Class ignoredClass : objectClassesToIgnore) {
                if (ignoredClass.isInstance(comp)) {
                    return;
                }
            }
        }
        if (instancesToIgnore != null) {
            for (Component ignoredComponent : instancesToIgnore) {
                if (ignoredComponent == comp) {
                    return;
                }
            }
        }

        if (!(comp instanceof RecursiveBackgroundNoRecursion) && comp instanceof Container) {
            Component[] components = ((Container) comp).getComponents();
            for (Component component : components) {
                setBackgroundRecursive(component, color, objectClassesToIgnore, instancesToIgnore);
            }
        }
        comp.setBackground(color);
    }

    public static JPanel getWrapperPanel(Component contained, boolean stretchToWrapper) {
        JPanel panelWrapper = new JPanel();
        if (stretchToWrapper) {
            panelWrapper.setLayout(new BorderLayout());
            panelWrapper.add(contained, BorderLayout.CENTER);
        } else {
            panelWrapper.add(contained);
        }
        return panelWrapper;
    }

    public static JPanel getWrapperPanel(Component contained) {
        return getWrapperPanel(contained, true);
    }

    public static JToolBar getToolBar() {
        return getToolBar(JToolBar.HORIZONTAL);
    }

    public static JToolBar getToolBar(int orientation) {
        JToolBar toolBar = new JToolBar(orientation);
        toolBar.setFloatable(false);
        return toolBar;
    }

    /**
     * Methods for obtaining a single usage JFrame, used as a Dialog.
     */
    public static JFrame getTemporaryDialogFrame(Container parent, Container dialogContents, String title, Image topIcon,
            boolean alwaysOnTop, boolean doPackAndCenter, boolean show) {
        JFrame temporaryFrame = new JFrame(title);
        temporaryFrame.setAlwaysOnTop(alwaysOnTop);
        //temporaryFrame.setBackground(GuiConfiguration.COLOR_PANEL_CONTAINER_BACK);
        configureTemporaryContainer(temporaryFrame, temporaryFrame, parent, dialogContents, title, topIcon, doPackAndCenter, show);
        return temporaryFrame;
    }

    public static JDialog getTemporaryDialog(JFrame frameOwner, Container dialogContents, String title, Image topIcon,
            boolean modal, boolean doPackAndCenter, boolean show) {
        JDialog temporaryDialog = new JDialog(frameOwner, title, modal);
        temporaryDialog.setIconImage(topIcon);
        //temporaryDialog.setBackground(GuiConfiguration.COLOR_PANEL_CONTAINER_BACK);
        configureTemporaryContainer(temporaryDialog, temporaryDialog, frameOwner, dialogContents, title, topIcon, doPackAndCenter, show);
        return temporaryDialog;
    }

    private static void configureTemporaryContainer(final Window temporaryContainer, RootPaneContainer sameContainer, Container parent,
            Container contents, String title, Image topIcon,
            boolean doPackAndCenter, boolean show) {
        temporaryContainer.setIconImage(topIcon);
        temporaryContainer.addWindowListener(new WindowAdapter() {

            @Override
            public void windowClosing(WindowEvent we) {
                temporaryContainer.dispose();
            }
        });

        sameContainer.getContentPane().add(contents);

        if (doPackAndCenter) {
            temporaryContainer.pack();
            if (parent == null) {
                centerComponentOnScreen(temporaryContainer);
            } else {
                centerComponentOnComponentParent(temporaryContainer, parent);
            }
        }
        if (show) {
            temporaryContainer.setVisible(true);
        }
    }
}
