/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package utils.dnd;

import java.awt.Component;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.InputEvent;
import javax.swing.JComponent;
import javax.swing.TransferHandler;
import utils.UtilsLogging;

/**
 *
 * Date: 23 Φεβ 2012
 * @author Jason Koukas
 */
public abstract class TransferHandlerCustomExtend extends TransferHandler {

    protected TransferHandler m_replacedHandler;
    private DataFlavor[] m_dropSupportedFlavors;
    private boolean m_validateTransferData;

    public static class DropTargetListenerCustom implements DropTargetListener {

        private DropTarget m_replacedTarget;

        public DropTargetListenerCustom(JComponent component) {
            m_replacedTarget = component.getDropTarget();
        }

        @Override
        public void drop(DropTargetDropEvent dtde) {
            //232:23:45,666
            //1232:23:45,666
            //System.out.println("drop");
            Component component = ((DropTarget) dtde.getSource()).getComponent();
            if (component instanceof JComponent) {
                dtde.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
                //m_replacedTarget.drop(dtde);
                dtde.dropComplete(((JComponent) component).getTransferHandler().importData((JComponent) component, dtde.getTransferable()));
            }
        }

        public void dragEnter(DropTargetDragEvent dtde) {
            m_replacedTarget.dragEnter(dtde);
        }

        public void dragOver(DropTargetDragEvent dtde) {
            m_replacedTarget.dragOver(dtde);
        }

        public void dropActionChanged(DropTargetDragEvent dtde) {
            m_replacedTarget.dropActionChanged(dtde);
        }

        public void dragExit(DropTargetEvent dte) {
            m_replacedTarget.dragExit(dte);
        }
    }

    public TransferHandlerCustomExtend(TransferHandler handlerReplaced, DataFlavor[] dropSupportedFlavors, boolean dataValidationEnabled) {
        m_replacedHandler = handlerReplaced;
        m_dropSupportedFlavors = dropSupportedFlavors;
        m_validateTransferData = dataValidationEnabled;
    }

    public TransferHandlerCustomExtend(JComponent component, DataFlavor[] dropSupportedFlavors, boolean dataValidationEnabled) {
        this(component.getTransferHandler(), dropSupportedFlavors, dataValidationEnabled);
        component.setTransferHandler(this);
        new DropTarget(component, new DropTargetListenerCustom(component));
    }

    public TransferHandlerCustomExtend(DataFlavor[] dropSupportedFlavors, boolean dataValidationEnabled) {
        this((TransferHandler) null, dropSupportedFlavors, dataValidationEnabled);
    }

    public void enableTransferDataValidation(boolean value) {
        m_validateTransferData = value;
    }

    protected abstract boolean validateTransferData(Transferable transferable, Object transferData, DataFlavor flavorPreferred);

    protected abstract boolean importData(Component comp, Transferable transferable, Object transferData, DataFlavor usedFlavor);

    /**
     * Methods Overriden from TransferHandler (for drop)
     */
    @Override
    public boolean canImport(TransferSupport support) {
        return canImportWhenTransferableIsGiven(support.getTransferable(), support.getDataFlavors());
    }

    @Override
    public boolean canImport(JComponent comp, DataFlavor[] transferableDropFlavors) {
        return getPreferredFlavor(transferableDropFlavors, m_dropSupportedFlavors) != null;
    }

    @Override
    public boolean importData(TransferSupport support) {
        return importData(support.getComponent(), support.getTransferable(), support.getDataFlavors());
    }

    @Override
    public boolean importData(JComponent comp, Transferable transferable) {
        return importData((Component) comp, transferable, transferable.getTransferDataFlavors());
    }

    /**
     * Other Methods Overriden from TransferHandler
     */
    @Override
    public int getSourceActions(JComponent c) {
//        return DnDConstants.ACTION_COPY_OR_MOVE;
        if (m_replacedHandler == null) {
            return super.getSourceActions(c);
        }
        return m_replacedHandler.getSourceActions(c);
    }

    @Override
    public void exportToClipboard(JComponent comp, Clipboard clip, int action)
            throws IllegalStateException {
        if (m_replacedHandler == null) {
            super.exportToClipboard(comp, clip, action);
            return;
        }
        m_replacedHandler.exportToClipboard(comp, clip, action);
    }

    @Override
    public void exportAsDrag(JComponent comp, InputEvent e, int action) {
        if (m_replacedHandler == null) {
            super.exportAsDrag(comp, e, action);
            return;
        }
        m_replacedHandler.exportAsDrag(comp, e, action);
    }

    /**
     * Private Helpful Methods
     */
    private boolean importData(Component comp, Transferable transferable, DataFlavor[] providedFlavors) {
        DataFlavor flavorPreferred = getPreferredFlavor(providedFlavors, m_dropSupportedFlavors);
        if (flavorPreferred == null) {
            return false;
        }
        Object transferData = getValidatedTransferData(transferable, flavorPreferred);
        if (transferData == null) {
            return false;
        }
        return importData(comp, transferable, transferData, flavorPreferred);
    }

    private boolean canImportWhenTransferableIsGiven(Transferable transferable, DataFlavor[] providedFlavors) {
        DataFlavor flavorPreferred = getPreferredFlavor(providedFlavors, m_dropSupportedFlavors);
        if (flavorPreferred == null) {
            return false;
        }
        Object transferData = getValidatedTransferData(transferable, flavorPreferred);
        if (transferData == null) {
            return false;
        }
        return true;
    }

    private Object getValidatedTransferData(Transferable transferable, DataFlavor flavorPreferred) {
        try {
            Object transferData = transferable.getTransferData(flavorPreferred);
            if (transferData == null) {
                return null;
            }
            if (m_validateTransferData && !validateTransferData(transferable, transferData, flavorPreferred)) {
                return null;
            }
            return transferData;
        } catch (Exception ex) {
            UtilsLogging.log(TransferHandlerCustomExtend.class, ex);
            return null;
        }
    }

    private static DataFlavor getPreferredFlavor(DataFlavor[] providedFlavors, DataFlavor[] supportedFlavors) {
        for (DataFlavor flavorProvided : providedFlavors) {
            if (isDataFlavorSupported(flavorProvided, supportedFlavors)) {
                return flavorProvided;
            }
        }
        return null;
    }

    public static boolean isDataFlavorSupported(DataFlavor flavorProvided, DataFlavor[] supportedFlavors) {
        for (DataFlavor flavorSupported : supportedFlavors) {
            if (flavorSupported.equals(flavorProvided)) {
                return true;
            }
        }
        return false;
    }
}
