/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package utils.ui;

import java.util.ArrayList;
import javax.swing.JFormattedTextField;
import javax.swing.JFormattedTextField.AbstractFormatter;
import javax.swing.JSpinner;
import javax.swing.SpinnerModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 *
 * Date: 18 Μαρ 2012
 * @author Jason Koukas
 */
public class JSpinnerEditable extends JSpinner {

    public JSpinnerEditable(SpinnerModel model) {
        super(model);
        init();
    }

    public JSpinnerEditable() {
        super();
        init();
    }

    private void init() {
        getFormattedTextField().setEditable(true);
    }

    public JFormattedTextField getFormattedTextField() {
        return (JFormattedTextField) getEditor().getComponent(0);
    }

    public void setTextFieldEditFormatter(final AbstractFormatter formatter) {
        getFormattedTextField().setFormatterFactory(new JFormattedTextField.AbstractFormatterFactory() {

            @Override
            public AbstractFormatter getFormatter(JFormattedTextField tf) {
                return formatter;
            }
        });
    }

    public static abstract class SpinnerModelCustom implements SpinnerModel {

        protected ArrayList<ChangeListener> m_changeListeners;

        @Override
        public void addChangeListener(ChangeListener l) {
            if (m_changeListeners == null) {
                m_changeListeners = new ArrayList<ChangeListener>();
            }
            m_changeListeners.add(l);
        }

        @Override
        public void removeChangeListener(ChangeListener l) {
            if (m_changeListeners != null) {
                m_changeListeners.remove(l);
            }
        }

        protected void notifyChangeListeners() {
            if (m_changeListeners != null) {
                ChangeEvent evt = new ChangeEvent(this);
                for (ChangeListener c : m_changeListeners) {
                    c.stateChanged(evt);
                }
            }
        }
    }

    public static class SpinnerModelInt extends SpinnerModelCustom {

        private int m_min;
        private int m_max;
        private int m_curr;

        public SpinnerModelInt(int minValue, int maxValue, int currVal) {
            if (minValue > maxValue) {
                throw new IllegalArgumentException("Min value must be lower or equal to max value.");
            }
            m_min = minValue;
            m_max = maxValue;

            setValue(currVal);
        }

        public int getMaxValue() {
            return m_max;
        }

        public int getMinValue() {
            return m_min;
        }

        public Object getValue() {
            return m_curr;
        }

        public void setValue(Object value) {
            if (value instanceof Integer) {
                int valueInt = (Integer) value;
                if (valueInt < m_min || valueInt > m_max) {
                    throw new IllegalArgumentException("Spinner value must be between min value and max value inclusive.");
                }
                m_curr = valueInt;
                notifyChangeListeners();
            } else {
                throw new IllegalArgumentException("Value must be of Integer type.");
            }
        }

        public Object getNextValue() {
            if (m_curr == m_max) {
                return m_min;
            }
            return m_curr + 1;
        }

        public Object getPreviousValue() {
            if (m_curr == m_min) {
                return m_max;
            }
            return m_curr - 1;
        }
    }
}
