/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package utils.ui;

import utils.math.DoubleLimit;
import utils.ui.JSpinnerEditable.SpinnerModelCustom;

/**
 *
 * Date: 31 Μαρ 2012
 * @author Jason Koukas
 */
public class SpinnerModelDoublePredefinedValuesCycle extends SpinnerModelCustom {

    private DoubleLimit m_min;
    private DoubleLimit m_max;
    private double m_curr;
    private double[] m_predefinedSuggestions;

    public SpinnerModelDoublePredefinedValuesCycle(double minValue, double maxValue, double currVal, double[] predefinedSuggestions) {
        this(new DoubleLimit(minValue), new DoubleLimit(maxValue), currVal, predefinedSuggestions);
    }

    public SpinnerModelDoublePredefinedValuesCycle(DoubleLimit minValue, DoubleLimit maxValue, double currVal, double[] predefinedSuggestions) {
        if (minValue.compareTo(maxValue) > 0) {
            throw new IllegalArgumentException("Min value must be lower or equal to max value.");
        }
        if (predefinedSuggestions == null) {
            throw new NullPointerException("Predefined values array can't be null.");
        }

        m_min = minValue;
        m_max = maxValue;

        double previousSuggestionValue = minValue.doubleValue() - 1;
        for (double value : predefinedSuggestions) {
            checkValueWithinBounds(value);
            if (value <= previousSuggestionValue) {
                throw new IllegalArgumentException("Predefined values must be ascending in array.");
            }
            previousSuggestionValue = value;
        }

        m_predefinedSuggestions = predefinedSuggestions;
        setValue(currVal);
    }

    public DoubleLimit getMaxValue() {
        return m_max;
    }

    public DoubleLimit getMinValue() {
        return m_min;
    }

    public Object getValue() {
        return m_curr;
    }

    public void setValue(Object value) {
        if (value instanceof Double) {
            double valueDouble = (Double) value;
            checkValueWithinBounds(valueDouble);
            m_curr = valueDouble;
            notifyChangeListeners();
        } else {
            throw new IllegalArgumentException("Value must be of Double type.");
        }
    }

    public Object getNextValue() {
        int i = 0;
        for (double value : m_predefinedSuggestions) {
            if (m_curr < value) {
                return value;
            }
            i++;
        }
        return m_predefinedSuggestions[0];
    }

    public Object getPreviousValue() {
        int i = m_predefinedSuggestions.length - 1;
        for (; i >= 0; i--) {
            double value = m_predefinedSuggestions[i];
            if (m_curr > value) {
                return value;
            }
        }
        return m_predefinedSuggestions[m_predefinedSuggestions.length - 1];
    }

    private void checkValueWithinBounds(double valueDouble) {
        if (m_min.compareTo(valueDouble) > 0 || m_max.compareTo(valueDouble) < 0) {
            throw new IllegalArgumentException("Values must be between min value and max value inclusive.");
        }
    }
}
